#include "atlas_misc.h"
#ifndef PFYDIST
   #define PFYDIST 16
#endif
#ifndef PFXDIST
   #define PFXDIST 0
#endif
#ifndef PFADIST
   #define PFADIST 0
#endif

#if !defined(ATL_3DNow) && !defined(ATL_SSE1) && \
    (PFADIST != 0 || PFXDIST != 0 || PFYDIST != 0)
   #include "atlas_prefetch.h"
#endif

#if defined(ATL_3DNow) || defined(ATL_SSE1)
   #ifndef PFIY
      #define PFIY prefetchnta
   #endif
   #ifndef PFIX
      #define PFIX prefetchnta
   #endif
   #ifndef PFIA
      #ifdef ATL_3DNow
         #define PFIA prefetchw
      #else
         #define PFIA prefetcht0
      #endif
   #endif
#endif
/*
 * X & A are prefetched in M loop PF[A,X]DIST (in bytes) ahead
 */
#if PFADIST == 0                /* flag for no prefetch */
   #define prefA(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefA(mem) __asm__ __volatile__ \
         (Mstr(PFIA) " %0" : : "m" (*(((char *)(mem))+PFADIST)))
   #else
      #if PFLVL == 2
         #define prefA(mem) ATL_pfl2W(((char*)mem)+PFADIST)
      #else
         #define prefA(mem) ATL_pfl1W(((char*)mem)+PFADIST)
      #endif
   #endif
#endif
#if PFXDIST == 0                /* flag for no prefetch */
   #define prefX(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefX(mem) __asm__ __volatile__ \
         (Mstr(PFIX) " %0" : : "m" (*(((char *)(mem))+PFXDIST)))
   #else
      #if PFLVL == 2
         #define prefX(mem) ATL_pfl2R(((char*)mem)+PFXDIST)
      #else
         #define prefX(mem) ATL_pfl1R(((char*)mem)+PFXDIST)
      #endif
   #endif
#endif
/*
 * Y is prefetched in N-loop, and always fetches next NU piece
 */
#if PFYDIST == 0                /* flag for no prefetch */
   #define prefY(mem)
#else
   #if defined(ATL_3DNow) || defined(ATL_SSE1)
      #define prefY(mem) \
         __asm__ __volatile__ (Mstr(PFIY) " %0" : : "m" (*(((char *)(mem)))))
   #else
      #if PFLVL == 2
         #define prefY(mem) ATL_pfl2R(mem)
      #else
         #define prefY(mem) ATL_pfl1R(mem)
      #endif
   #endif
#endif
#ifndef ATL_CINT
   #define ATL_CINT const int
#endif
#ifndef ATL_INT
   #define ATL_INT int
#endif
void ATL_UGER2K(ATL_CINT M, ATL_CINT N, const float *X, const float *Y, 
                const float *W, const float *Z, float *A, ATL_CINT lda)
/*
 * 48x16 unrolled r2_c.
 * Extracted from ATLAS/tune/blas/gemv/atlas-l2g.base
 */
{
   ATL_CINT N16=(N/16)*16, M48=(M/48)*48, lda16=lda*16;
   ATL_INT j;

   for (j=N16; j; j -= 16, A += lda16, Y += 16)
   {
      float *A0=A, *A1=A0+lda, *A2=A1+lda, *A3=A2+lda, *A4=A3+lda, *A5=A4+lda,
            *A6=A5+lda, *A7=A6+lda, *A8=A7+lda, *A9=A8+lda, *A10=A9+lda,
            *A11=A10+lda, *A12=A11+lda, *A13=A12+lda, *A14=A13+lda,
            *A15=A14+lda;
      const register float y0=Y[0], z0=Z[0], y1=Y[1], z1=Z[1], y2=Y[2], z2=Z[2],
                           y3=Y[3], z3=Z[3], y4=Y[4], z4=Z[4], y5=Y[5], z5=Z[5],
                           y6=Y[6], z6=Z[6], y7=Y[7], z7=Z[7], y8=Y[8], z8=Z[8],
                           y9=Y[9], z9=Z[9], y10=Y[10], z10=Z[10], y11=Y[11],
                           z11=Z[11], y12=Y[12], z12=Z[12], y13=Y[13],
                           z13=Z[13], y14=Y[14], z14=Z[14], y15=Y[15],
                           z15=Z[15];
      ATL_INT i;
      prefY(Y+16+16-1);
      prefY(Z+16+16-1);
      for (i=0; i < M48; i += 48)
      {
         register float a0, a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12,
                        a13, a14, a15;
         const float x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                     w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                     x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                     w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                     x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                     x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                     x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15],
                     x16=X[i+16], w16=W[i+16], x17=X[i+17], w17=W[i+17],
                     x18=X[i+18], w18=W[i+18], x19=X[i+19], w19=W[i+19],
                     x20=X[i+20], w20=W[i+20], x21=X[i+21], w21=W[i+21],
                     x22=X[i+22], w22=W[i+22], x23=X[i+23], w23=W[i+23],
                     x24=X[i+24], w24=W[i+24], x25=X[i+25], w25=W[i+25],
                     x26=X[i+26], w26=W[i+26], x27=X[i+27], w27=W[i+27],
                     x28=X[i+28], w28=W[i+28], x29=X[i+29], w29=W[i+29],
                     x30=X[i+30], w30=W[i+30], x31=X[i+31], w31=W[i+31],
                     x32=X[i+32], w32=W[i+32], x33=X[i+33], w33=W[i+33],
                     x34=X[i+34], w34=W[i+34], x35=X[i+35], w35=W[i+35],
                     x36=X[i+36], w36=W[i+36], x37=X[i+37], w37=W[i+37],
                     x38=X[i+38], w38=W[i+38], x39=X[i+39], w39=W[i+39],
                     x40=X[i+40], w40=W[i+40], x41=X[i+41], w41=W[i+41],
                     x42=X[i+42], w42=W[i+42], x43=X[i+43], w43=W[i+43],
                     x44=X[i+44], w44=W[i+44], x45=X[i+45], w45=W[i+45],
                     x46=X[i+46], w46=W[i+46], x47=X[i+47], w47=W[i+47];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         a1 = A1[i+0];
         a1 += x0 * y1;
         a2 = A2[i+0];
         a2 += x0 * y2;
         a3 = A3[i+0];
         a3 += x0 * y3;
         a4 = A4[i+0];
         a4 += x0 * y4;
         a5 = A5[i+0];
         a5 += x0 * y5;
         a6 = A6[i+0];
         a6 += x0 * y6;
         a7 = A7[i+0];
         a7 += x0 * y7;
         a8 = A8[i+0];
         a8 += x0 * y8;
         a9 = A9[i+0];
         a9 += x0 * y9;
         a10 = A10[i+0];
         a10 += x0 * y10;
         a11 = A11[i+0];
         a11 += x0 * y11;
         a12 = A12[i+0];
         a12 += x0 * y12;
         a13 = A13[i+0];
         a13 += x0 * y13;
         a14 = A14[i+0];
         a14 += x0 * y14;
         a15 = A15[i+0];
         a15 += x0 * y15;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a1 += w0 * z1;
         prefA(A1);
         A1[i+0] = a1;
         a2 += w0 * z2;
         prefA(A2);
         A2[i+0] = a2;
         a3 += w0 * z3;
         prefA(A3);
         A3[i+0] = a3;
         a4 += w0 * z4;
         prefA(A4);
         A4[i+0] = a4;
         a5 += w0 * z5;
         prefA(A5);
         A5[i+0] = a5;
         a6 += w0 * z6;
         prefA(A6);
         A6[i+0] = a6;
         a7 += w0 * z7;
         prefA(A7);
         A7[i+0] = a7;
         a8 += w0 * z8;
         prefA(A8);
         A8[i+0] = a8;
         a9 += w0 * z9;
         prefA(A9);
         A9[i+0] = a9;
         a10 += w0 * z10;
         prefA(A10);
         A10[i+0] = a10;
         a11 += w0 * z11;
         prefA(A11);
         A11[i+0] = a11;
         a12 += w0 * z12;
         prefA(A12);
         A12[i+0] = a12;
         a13 += w0 * z13;
         prefA(A13);
         A13[i+0] = a13;
         a14 += w0 * z14;
         prefA(A14);
         A14[i+0] = a14;
         a15 += w0 * z15;
         prefA(A15);
         A15[i+0] = a15;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a1 = A1[i+1];
         a1 += x1 * y1;
         a2 = A2[i+1];
         a2 += x1 * y2;
         a3 = A3[i+1];
         a3 += x1 * y3;
         a4 = A4[i+1];
         a4 += x1 * y4;
         a5 = A5[i+1];
         a5 += x1 * y5;
         a6 = A6[i+1];
         a6 += x1 * y6;
         a7 = A7[i+1];
         a7 += x1 * y7;
         a8 = A8[i+1];
         a8 += x1 * y8;
         a9 = A9[i+1];
         a9 += x1 * y9;
         a10 = A10[i+1];
         a10 += x1 * y10;
         a11 = A11[i+1];
         a11 += x1 * y11;
         a12 = A12[i+1];
         a12 += x1 * y12;
         a13 = A13[i+1];
         a13 += x1 * y13;
         a14 = A14[i+1];
         a14 += x1 * y14;
         a15 = A15[i+1];
         a15 += x1 * y15;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a1 += w1 * z1;
         A1[i+1] = a1;
         a2 += w1 * z2;
         A2[i+1] = a2;
         a3 += w1 * z3;
         A3[i+1] = a3;
         a4 += w1 * z4;
         A4[i+1] = a4;
         a5 += w1 * z5;
         A5[i+1] = a5;
         a6 += w1 * z6;
         A6[i+1] = a6;
         a7 += w1 * z7;
         A7[i+1] = a7;
         a8 += w1 * z8;
         A8[i+1] = a8;
         a9 += w1 * z9;
         A9[i+1] = a9;
         a10 += w1 * z10;
         A10[i+1] = a10;
         a11 += w1 * z11;
         A11[i+1] = a11;
         a12 += w1 * z12;
         A12[i+1] = a12;
         a13 += w1 * z13;
         A13[i+1] = a13;
         a14 += w1 * z14;
         A14[i+1] = a14;
         a15 += w1 * z15;
         A15[i+1] = a15;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a1 = A1[i+2];
         a1 += x2 * y1;
         a2 = A2[i+2];
         a2 += x2 * y2;
         a3 = A3[i+2];
         a3 += x2 * y3;
         a4 = A4[i+2];
         a4 += x2 * y4;
         a5 = A5[i+2];
         a5 += x2 * y5;
         a6 = A6[i+2];
         a6 += x2 * y6;
         a7 = A7[i+2];
         a7 += x2 * y7;
         a8 = A8[i+2];
         a8 += x2 * y8;
         a9 = A9[i+2];
         a9 += x2 * y9;
         a10 = A10[i+2];
         a10 += x2 * y10;
         a11 = A11[i+2];
         a11 += x2 * y11;
         a12 = A12[i+2];
         a12 += x2 * y12;
         a13 = A13[i+2];
         a13 += x2 * y13;
         a14 = A14[i+2];
         a14 += x2 * y14;
         a15 = A15[i+2];
         a15 += x2 * y15;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a1 += w2 * z1;
         A1[i+2] = a1;
         a2 += w2 * z2;
         A2[i+2] = a2;
         a3 += w2 * z3;
         A3[i+2] = a3;
         a4 += w2 * z4;
         A4[i+2] = a4;
         a5 += w2 * z5;
         A5[i+2] = a5;
         a6 += w2 * z6;
         A6[i+2] = a6;
         a7 += w2 * z7;
         A7[i+2] = a7;
         a8 += w2 * z8;
         A8[i+2] = a8;
         a9 += w2 * z9;
         A9[i+2] = a9;
         a10 += w2 * z10;
         A10[i+2] = a10;
         a11 += w2 * z11;
         A11[i+2] = a11;
         a12 += w2 * z12;
         A12[i+2] = a12;
         a13 += w2 * z13;
         A13[i+2] = a13;
         a14 += w2 * z14;
         A14[i+2] = a14;
         a15 += w2 * z15;
         A15[i+2] = a15;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a1 = A1[i+3];
         a1 += x3 * y1;
         a2 = A2[i+3];
         a2 += x3 * y2;
         a3 = A3[i+3];
         a3 += x3 * y3;
         a4 = A4[i+3];
         a4 += x3 * y4;
         a5 = A5[i+3];
         a5 += x3 * y5;
         a6 = A6[i+3];
         a6 += x3 * y6;
         a7 = A7[i+3];
         a7 += x3 * y7;
         a8 = A8[i+3];
         a8 += x3 * y8;
         a9 = A9[i+3];
         a9 += x3 * y9;
         a10 = A10[i+3];
         a10 += x3 * y10;
         a11 = A11[i+3];
         a11 += x3 * y11;
         a12 = A12[i+3];
         a12 += x3 * y12;
         a13 = A13[i+3];
         a13 += x3 * y13;
         a14 = A14[i+3];
         a14 += x3 * y14;
         a15 = A15[i+3];
         a15 += x3 * y15;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a1 += w3 * z1;
         A1[i+3] = a1;
         a2 += w3 * z2;
         A2[i+3] = a2;
         a3 += w3 * z3;
         A3[i+3] = a3;
         a4 += w3 * z4;
         A4[i+3] = a4;
         a5 += w3 * z5;
         A5[i+3] = a5;
         a6 += w3 * z6;
         A6[i+3] = a6;
         a7 += w3 * z7;
         A7[i+3] = a7;
         a8 += w3 * z8;
         A8[i+3] = a8;
         a9 += w3 * z9;
         A9[i+3] = a9;
         a10 += w3 * z10;
         A10[i+3] = a10;
         a11 += w3 * z11;
         A11[i+3] = a11;
         a12 += w3 * z12;
         A12[i+3] = a12;
         a13 += w3 * z13;
         A13[i+3] = a13;
         a14 += w3 * z14;
         A14[i+3] = a14;
         a15 += w3 * z15;
         A15[i+3] = a15;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a1 = A1[i+4];
         a1 += x4 * y1;
         a2 = A2[i+4];
         a2 += x4 * y2;
         a3 = A3[i+4];
         a3 += x4 * y3;
         a4 = A4[i+4];
         a4 += x4 * y4;
         a5 = A5[i+4];
         a5 += x4 * y5;
         a6 = A6[i+4];
         a6 += x4 * y6;
         a7 = A7[i+4];
         a7 += x4 * y7;
         a8 = A8[i+4];
         a8 += x4 * y8;
         a9 = A9[i+4];
         a9 += x4 * y9;
         a10 = A10[i+4];
         a10 += x4 * y10;
         a11 = A11[i+4];
         a11 += x4 * y11;
         a12 = A12[i+4];
         a12 += x4 * y12;
         a13 = A13[i+4];
         a13 += x4 * y13;
         a14 = A14[i+4];
         a14 += x4 * y14;
         a15 = A15[i+4];
         a15 += x4 * y15;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a1 += w4 * z1;
         A1[i+4] = a1;
         a2 += w4 * z2;
         A2[i+4] = a2;
         a3 += w4 * z3;
         A3[i+4] = a3;
         a4 += w4 * z4;
         A4[i+4] = a4;
         a5 += w4 * z5;
         A5[i+4] = a5;
         a6 += w4 * z6;
         A6[i+4] = a6;
         a7 += w4 * z7;
         A7[i+4] = a7;
         a8 += w4 * z8;
         A8[i+4] = a8;
         a9 += w4 * z9;
         A9[i+4] = a9;
         a10 += w4 * z10;
         A10[i+4] = a10;
         a11 += w4 * z11;
         A11[i+4] = a11;
         a12 += w4 * z12;
         A12[i+4] = a12;
         a13 += w4 * z13;
         A13[i+4] = a13;
         a14 += w4 * z14;
         A14[i+4] = a14;
         a15 += w4 * z15;
         A15[i+4] = a15;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a1 = A1[i+5];
         a1 += x5 * y1;
         a2 = A2[i+5];
         a2 += x5 * y2;
         a3 = A3[i+5];
         a3 += x5 * y3;
         a4 = A4[i+5];
         a4 += x5 * y4;
         a5 = A5[i+5];
         a5 += x5 * y5;
         a6 = A6[i+5];
         a6 += x5 * y6;
         a7 = A7[i+5];
         a7 += x5 * y7;
         a8 = A8[i+5];
         a8 += x5 * y8;
         a9 = A9[i+5];
         a9 += x5 * y9;
         a10 = A10[i+5];
         a10 += x5 * y10;
         a11 = A11[i+5];
         a11 += x5 * y11;
         a12 = A12[i+5];
         a12 += x5 * y12;
         a13 = A13[i+5];
         a13 += x5 * y13;
         a14 = A14[i+5];
         a14 += x5 * y14;
         a15 = A15[i+5];
         a15 += x5 * y15;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a1 += w5 * z1;
         A1[i+5] = a1;
         a2 += w5 * z2;
         A2[i+5] = a2;
         a3 += w5 * z3;
         A3[i+5] = a3;
         a4 += w5 * z4;
         A4[i+5] = a4;
         a5 += w5 * z5;
         A5[i+5] = a5;
         a6 += w5 * z6;
         A6[i+5] = a6;
         a7 += w5 * z7;
         A7[i+5] = a7;
         a8 += w5 * z8;
         A8[i+5] = a8;
         a9 += w5 * z9;
         A9[i+5] = a9;
         a10 += w5 * z10;
         A10[i+5] = a10;
         a11 += w5 * z11;
         A11[i+5] = a11;
         a12 += w5 * z12;
         A12[i+5] = a12;
         a13 += w5 * z13;
         A13[i+5] = a13;
         a14 += w5 * z14;
         A14[i+5] = a14;
         a15 += w5 * z15;
         A15[i+5] = a15;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a1 = A1[i+6];
         a1 += x6 * y1;
         a2 = A2[i+6];
         a2 += x6 * y2;
         a3 = A3[i+6];
         a3 += x6 * y3;
         a4 = A4[i+6];
         a4 += x6 * y4;
         a5 = A5[i+6];
         a5 += x6 * y5;
         a6 = A6[i+6];
         a6 += x6 * y6;
         a7 = A7[i+6];
         a7 += x6 * y7;
         a8 = A8[i+6];
         a8 += x6 * y8;
         a9 = A9[i+6];
         a9 += x6 * y9;
         a10 = A10[i+6];
         a10 += x6 * y10;
         a11 = A11[i+6];
         a11 += x6 * y11;
         a12 = A12[i+6];
         a12 += x6 * y12;
         a13 = A13[i+6];
         a13 += x6 * y13;
         a14 = A14[i+6];
         a14 += x6 * y14;
         a15 = A15[i+6];
         a15 += x6 * y15;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a1 += w6 * z1;
         A1[i+6] = a1;
         a2 += w6 * z2;
         A2[i+6] = a2;
         a3 += w6 * z3;
         A3[i+6] = a3;
         a4 += w6 * z4;
         A4[i+6] = a4;
         a5 += w6 * z5;
         A5[i+6] = a5;
         a6 += w6 * z6;
         A6[i+6] = a6;
         a7 += w6 * z7;
         A7[i+6] = a7;
         a8 += w6 * z8;
         A8[i+6] = a8;
         a9 += w6 * z9;
         A9[i+6] = a9;
         a10 += w6 * z10;
         A10[i+6] = a10;
         a11 += w6 * z11;
         A11[i+6] = a11;
         a12 += w6 * z12;
         A12[i+6] = a12;
         a13 += w6 * z13;
         A13[i+6] = a13;
         a14 += w6 * z14;
         A14[i+6] = a14;
         a15 += w6 * z15;
         A15[i+6] = a15;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a1 = A1[i+7];
         a1 += x7 * y1;
         a2 = A2[i+7];
         a2 += x7 * y2;
         a3 = A3[i+7];
         a3 += x7 * y3;
         a4 = A4[i+7];
         a4 += x7 * y4;
         a5 = A5[i+7];
         a5 += x7 * y5;
         a6 = A6[i+7];
         a6 += x7 * y6;
         a7 = A7[i+7];
         a7 += x7 * y7;
         a8 = A8[i+7];
         a8 += x7 * y8;
         a9 = A9[i+7];
         a9 += x7 * y9;
         a10 = A10[i+7];
         a10 += x7 * y10;
         a11 = A11[i+7];
         a11 += x7 * y11;
         a12 = A12[i+7];
         a12 += x7 * y12;
         a13 = A13[i+7];
         a13 += x7 * y13;
         a14 = A14[i+7];
         a14 += x7 * y14;
         a15 = A15[i+7];
         a15 += x7 * y15;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a1 += w7 * z1;
         A1[i+7] = a1;
         a2 += w7 * z2;
         A2[i+7] = a2;
         a3 += w7 * z3;
         A3[i+7] = a3;
         a4 += w7 * z4;
         A4[i+7] = a4;
         a5 += w7 * z5;
         A5[i+7] = a5;
         a6 += w7 * z6;
         A6[i+7] = a6;
         a7 += w7 * z7;
         A7[i+7] = a7;
         a8 += w7 * z8;
         A8[i+7] = a8;
         a9 += w7 * z9;
         A9[i+7] = a9;
         a10 += w7 * z10;
         A10[i+7] = a10;
         a11 += w7 * z11;
         A11[i+7] = a11;
         a12 += w7 * z12;
         A12[i+7] = a12;
         a13 += w7 * z13;
         A13[i+7] = a13;
         a14 += w7 * z14;
         A14[i+7] = a14;
         a15 += w7 * z15;
         A15[i+7] = a15;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a1 = A1[i+8];
         a1 += x8 * y1;
         a2 = A2[i+8];
         a2 += x8 * y2;
         a3 = A3[i+8];
         a3 += x8 * y3;
         a4 = A4[i+8];
         a4 += x8 * y4;
         a5 = A5[i+8];
         a5 += x8 * y5;
         a6 = A6[i+8];
         a6 += x8 * y6;
         a7 = A7[i+8];
         a7 += x8 * y7;
         a8 = A8[i+8];
         a8 += x8 * y8;
         a9 = A9[i+8];
         a9 += x8 * y9;
         a10 = A10[i+8];
         a10 += x8 * y10;
         a11 = A11[i+8];
         a11 += x8 * y11;
         a12 = A12[i+8];
         a12 += x8 * y12;
         a13 = A13[i+8];
         a13 += x8 * y13;
         a14 = A14[i+8];
         a14 += x8 * y14;
         a15 = A15[i+8];
         a15 += x8 * y15;
         a0 += w8 * z0;
         A0[i+8] = a0;
         a1 += w8 * z1;
         A1[i+8] = a1;
         a2 += w8 * z2;
         A2[i+8] = a2;
         a3 += w8 * z3;
         A3[i+8] = a3;
         a4 += w8 * z4;
         A4[i+8] = a4;
         a5 += w8 * z5;
         A5[i+8] = a5;
         a6 += w8 * z6;
         A6[i+8] = a6;
         a7 += w8 * z7;
         A7[i+8] = a7;
         a8 += w8 * z8;
         A8[i+8] = a8;
         a9 += w8 * z9;
         A9[i+8] = a9;
         a10 += w8 * z10;
         A10[i+8] = a10;
         a11 += w8 * z11;
         A11[i+8] = a11;
         a12 += w8 * z12;
         A12[i+8] = a12;
         a13 += w8 * z13;
         A13[i+8] = a13;
         a14 += w8 * z14;
         A14[i+8] = a14;
         a15 += w8 * z15;
         A15[i+8] = a15;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a1 = A1[i+9];
         a1 += x9 * y1;
         a2 = A2[i+9];
         a2 += x9 * y2;
         a3 = A3[i+9];
         a3 += x9 * y3;
         a4 = A4[i+9];
         a4 += x9 * y4;
         a5 = A5[i+9];
         a5 += x9 * y5;
         a6 = A6[i+9];
         a6 += x9 * y6;
         a7 = A7[i+9];
         a7 += x9 * y7;
         a8 = A8[i+9];
         a8 += x9 * y8;
         a9 = A9[i+9];
         a9 += x9 * y9;
         a10 = A10[i+9];
         a10 += x9 * y10;
         a11 = A11[i+9];
         a11 += x9 * y11;
         a12 = A12[i+9];
         a12 += x9 * y12;
         a13 = A13[i+9];
         a13 += x9 * y13;
         a14 = A14[i+9];
         a14 += x9 * y14;
         a15 = A15[i+9];
         a15 += x9 * y15;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a1 += w9 * z1;
         A1[i+9] = a1;
         a2 += w9 * z2;
         A2[i+9] = a2;
         a3 += w9 * z3;
         A3[i+9] = a3;
         a4 += w9 * z4;
         A4[i+9] = a4;
         a5 += w9 * z5;
         A5[i+9] = a5;
         a6 += w9 * z6;
         A6[i+9] = a6;
         a7 += w9 * z7;
         A7[i+9] = a7;
         a8 += w9 * z8;
         A8[i+9] = a8;
         a9 += w9 * z9;
         A9[i+9] = a9;
         a10 += w9 * z10;
         A10[i+9] = a10;
         a11 += w9 * z11;
         A11[i+9] = a11;
         a12 += w9 * z12;
         A12[i+9] = a12;
         a13 += w9 * z13;
         A13[i+9] = a13;
         a14 += w9 * z14;
         A14[i+9] = a14;
         a15 += w9 * z15;
         A15[i+9] = a15;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a1 = A1[i+10];
         a1 += x10 * y1;
         a2 = A2[i+10];
         a2 += x10 * y2;
         a3 = A3[i+10];
         a3 += x10 * y3;
         a4 = A4[i+10];
         a4 += x10 * y4;
         a5 = A5[i+10];
         a5 += x10 * y5;
         a6 = A6[i+10];
         a6 += x10 * y6;
         a7 = A7[i+10];
         a7 += x10 * y7;
         a8 = A8[i+10];
         a8 += x10 * y8;
         a9 = A9[i+10];
         a9 += x10 * y9;
         a10 = A10[i+10];
         a10 += x10 * y10;
         a11 = A11[i+10];
         a11 += x10 * y11;
         a12 = A12[i+10];
         a12 += x10 * y12;
         a13 = A13[i+10];
         a13 += x10 * y13;
         a14 = A14[i+10];
         a14 += x10 * y14;
         a15 = A15[i+10];
         a15 += x10 * y15;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a1 += w10 * z1;
         A1[i+10] = a1;
         a2 += w10 * z2;
         A2[i+10] = a2;
         a3 += w10 * z3;
         A3[i+10] = a3;
         a4 += w10 * z4;
         A4[i+10] = a4;
         a5 += w10 * z5;
         A5[i+10] = a5;
         a6 += w10 * z6;
         A6[i+10] = a6;
         a7 += w10 * z7;
         A7[i+10] = a7;
         a8 += w10 * z8;
         A8[i+10] = a8;
         a9 += w10 * z9;
         A9[i+10] = a9;
         a10 += w10 * z10;
         A10[i+10] = a10;
         a11 += w10 * z11;
         A11[i+10] = a11;
         a12 += w10 * z12;
         A12[i+10] = a12;
         a13 += w10 * z13;
         A13[i+10] = a13;
         a14 += w10 * z14;
         A14[i+10] = a14;
         a15 += w10 * z15;
         A15[i+10] = a15;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a1 = A1[i+11];
         a1 += x11 * y1;
         a2 = A2[i+11];
         a2 += x11 * y2;
         a3 = A3[i+11];
         a3 += x11 * y3;
         a4 = A4[i+11];
         a4 += x11 * y4;
         a5 = A5[i+11];
         a5 += x11 * y5;
         a6 = A6[i+11];
         a6 += x11 * y6;
         a7 = A7[i+11];
         a7 += x11 * y7;
         a8 = A8[i+11];
         a8 += x11 * y8;
         a9 = A9[i+11];
         a9 += x11 * y9;
         a10 = A10[i+11];
         a10 += x11 * y10;
         a11 = A11[i+11];
         a11 += x11 * y11;
         a12 = A12[i+11];
         a12 += x11 * y12;
         a13 = A13[i+11];
         a13 += x11 * y13;
         a14 = A14[i+11];
         a14 += x11 * y14;
         a15 = A15[i+11];
         a15 += x11 * y15;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a1 += w11 * z1;
         A1[i+11] = a1;
         a2 += w11 * z2;
         A2[i+11] = a2;
         a3 += w11 * z3;
         A3[i+11] = a3;
         a4 += w11 * z4;
         A4[i+11] = a4;
         a5 += w11 * z5;
         A5[i+11] = a5;
         a6 += w11 * z6;
         A6[i+11] = a6;
         a7 += w11 * z7;
         A7[i+11] = a7;
         a8 += w11 * z8;
         A8[i+11] = a8;
         a9 += w11 * z9;
         A9[i+11] = a9;
         a10 += w11 * z10;
         A10[i+11] = a10;
         a11 += w11 * z11;
         A11[i+11] = a11;
         a12 += w11 * z12;
         A12[i+11] = a12;
         a13 += w11 * z13;
         A13[i+11] = a13;
         a14 += w11 * z14;
         A14[i+11] = a14;
         a15 += w11 * z15;
         A15[i+11] = a15;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a1 = A1[i+12];
         a1 += x12 * y1;
         a2 = A2[i+12];
         a2 += x12 * y2;
         a3 = A3[i+12];
         a3 += x12 * y3;
         a4 = A4[i+12];
         a4 += x12 * y4;
         a5 = A5[i+12];
         a5 += x12 * y5;
         a6 = A6[i+12];
         a6 += x12 * y6;
         a7 = A7[i+12];
         a7 += x12 * y7;
         a8 = A8[i+12];
         a8 += x12 * y8;
         a9 = A9[i+12];
         a9 += x12 * y9;
         a10 = A10[i+12];
         a10 += x12 * y10;
         a11 = A11[i+12];
         a11 += x12 * y11;
         a12 = A12[i+12];
         a12 += x12 * y12;
         a13 = A13[i+12];
         a13 += x12 * y13;
         a14 = A14[i+12];
         a14 += x12 * y14;
         a15 = A15[i+12];
         a15 += x12 * y15;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a1 += w12 * z1;
         A1[i+12] = a1;
         a2 += w12 * z2;
         A2[i+12] = a2;
         a3 += w12 * z3;
         A3[i+12] = a3;
         a4 += w12 * z4;
         A4[i+12] = a4;
         a5 += w12 * z5;
         A5[i+12] = a5;
         a6 += w12 * z6;
         A6[i+12] = a6;
         a7 += w12 * z7;
         A7[i+12] = a7;
         a8 += w12 * z8;
         A8[i+12] = a8;
         a9 += w12 * z9;
         A9[i+12] = a9;
         a10 += w12 * z10;
         A10[i+12] = a10;
         a11 += w12 * z11;
         A11[i+12] = a11;
         a12 += w12 * z12;
         A12[i+12] = a12;
         a13 += w12 * z13;
         A13[i+12] = a13;
         a14 += w12 * z14;
         A14[i+12] = a14;
         a15 += w12 * z15;
         A15[i+12] = a15;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a1 = A1[i+13];
         a1 += x13 * y1;
         a2 = A2[i+13];
         a2 += x13 * y2;
         a3 = A3[i+13];
         a3 += x13 * y3;
         a4 = A4[i+13];
         a4 += x13 * y4;
         a5 = A5[i+13];
         a5 += x13 * y5;
         a6 = A6[i+13];
         a6 += x13 * y6;
         a7 = A7[i+13];
         a7 += x13 * y7;
         a8 = A8[i+13];
         a8 += x13 * y8;
         a9 = A9[i+13];
         a9 += x13 * y9;
         a10 = A10[i+13];
         a10 += x13 * y10;
         a11 = A11[i+13];
         a11 += x13 * y11;
         a12 = A12[i+13];
         a12 += x13 * y12;
         a13 = A13[i+13];
         a13 += x13 * y13;
         a14 = A14[i+13];
         a14 += x13 * y14;
         a15 = A15[i+13];
         a15 += x13 * y15;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a1 += w13 * z1;
         A1[i+13] = a1;
         a2 += w13 * z2;
         A2[i+13] = a2;
         a3 += w13 * z3;
         A3[i+13] = a3;
         a4 += w13 * z4;
         A4[i+13] = a4;
         a5 += w13 * z5;
         A5[i+13] = a5;
         a6 += w13 * z6;
         A6[i+13] = a6;
         a7 += w13 * z7;
         A7[i+13] = a7;
         a8 += w13 * z8;
         A8[i+13] = a8;
         a9 += w13 * z9;
         A9[i+13] = a9;
         a10 += w13 * z10;
         A10[i+13] = a10;
         a11 += w13 * z11;
         A11[i+13] = a11;
         a12 += w13 * z12;
         A12[i+13] = a12;
         a13 += w13 * z13;
         A13[i+13] = a13;
         a14 += w13 * z14;
         A14[i+13] = a14;
         a15 += w13 * z15;
         A15[i+13] = a15;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a1 = A1[i+14];
         a1 += x14 * y1;
         a2 = A2[i+14];
         a2 += x14 * y2;
         a3 = A3[i+14];
         a3 += x14 * y3;
         a4 = A4[i+14];
         a4 += x14 * y4;
         a5 = A5[i+14];
         a5 += x14 * y5;
         a6 = A6[i+14];
         a6 += x14 * y6;
         a7 = A7[i+14];
         a7 += x14 * y7;
         a8 = A8[i+14];
         a8 += x14 * y8;
         a9 = A9[i+14];
         a9 += x14 * y9;
         a10 = A10[i+14];
         a10 += x14 * y10;
         a11 = A11[i+14];
         a11 += x14 * y11;
         a12 = A12[i+14];
         a12 += x14 * y12;
         a13 = A13[i+14];
         a13 += x14 * y13;
         a14 = A14[i+14];
         a14 += x14 * y14;
         a15 = A15[i+14];
         a15 += x14 * y15;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a1 += w14 * z1;
         A1[i+14] = a1;
         a2 += w14 * z2;
         A2[i+14] = a2;
         a3 += w14 * z3;
         A3[i+14] = a3;
         a4 += w14 * z4;
         A4[i+14] = a4;
         a5 += w14 * z5;
         A5[i+14] = a5;
         a6 += w14 * z6;
         A6[i+14] = a6;
         a7 += w14 * z7;
         A7[i+14] = a7;
         a8 += w14 * z8;
         A8[i+14] = a8;
         a9 += w14 * z9;
         A9[i+14] = a9;
         a10 += w14 * z10;
         A10[i+14] = a10;
         a11 += w14 * z11;
         A11[i+14] = a11;
         a12 += w14 * z12;
         A12[i+14] = a12;
         a13 += w14 * z13;
         A13[i+14] = a13;
         a14 += w14 * z14;
         A14[i+14] = a14;
         a15 += w14 * z15;
         A15[i+14] = a15;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a1 = A1[i+15];
         a1 += x15 * y1;
         a2 = A2[i+15];
         a2 += x15 * y2;
         a3 = A3[i+15];
         a3 += x15 * y3;
         a4 = A4[i+15];
         a4 += x15 * y4;
         a5 = A5[i+15];
         a5 += x15 * y5;
         a6 = A6[i+15];
         a6 += x15 * y6;
         a7 = A7[i+15];
         a7 += x15 * y7;
         a8 = A8[i+15];
         a8 += x15 * y8;
         a9 = A9[i+15];
         a9 += x15 * y9;
         a10 = A10[i+15];
         a10 += x15 * y10;
         a11 = A11[i+15];
         a11 += x15 * y11;
         a12 = A12[i+15];
         a12 += x15 * y12;
         a13 = A13[i+15];
         a13 += x15 * y13;
         a14 = A14[i+15];
         a14 += x15 * y14;
         a15 = A15[i+15];
         a15 += x15 * y15;
         a0 += w15 * z0;
         A0[i+15] = a0;
         a1 += w15 * z1;
         A1[i+15] = a1;
         a2 += w15 * z2;
         A2[i+15] = a2;
         a3 += w15 * z3;
         A3[i+15] = a3;
         a4 += w15 * z4;
         A4[i+15] = a4;
         a5 += w15 * z5;
         A5[i+15] = a5;
         a6 += w15 * z6;
         A6[i+15] = a6;
         a7 += w15 * z7;
         A7[i+15] = a7;
         a8 += w15 * z8;
         A8[i+15] = a8;
         a9 += w15 * z9;
         A9[i+15] = a9;
         a10 += w15 * z10;
         A10[i+15] = a10;
         a11 += w15 * z11;
         A11[i+15] = a11;
         a12 += w15 * z12;
         A12[i+15] = a12;
         a13 += w15 * z13;
         A13[i+15] = a13;
         a14 += w15 * z14;
         A14[i+15] = a14;
         a15 += w15 * z15;
         A15[i+15] = a15;
         a0 = A0[i+16];
         a0 += x16 * y0;
         a1 = A1[i+16];
         a1 += x16 * y1;
         a2 = A2[i+16];
         a2 += x16 * y2;
         a3 = A3[i+16];
         a3 += x16 * y3;
         a4 = A4[i+16];
         a4 += x16 * y4;
         a5 = A5[i+16];
         a5 += x16 * y5;
         a6 = A6[i+16];
         a6 += x16 * y6;
         a7 = A7[i+16];
         a7 += x16 * y7;
         a8 = A8[i+16];
         a8 += x16 * y8;
         a9 = A9[i+16];
         a9 += x16 * y9;
         a10 = A10[i+16];
         a10 += x16 * y10;
         a11 = A11[i+16];
         a11 += x16 * y11;
         a12 = A12[i+16];
         a12 += x16 * y12;
         a13 = A13[i+16];
         a13 += x16 * y13;
         a14 = A14[i+16];
         a14 += x16 * y14;
         a15 = A15[i+16];
         a15 += x16 * y15;
         a0 += w16 * z0;
         prefA(A0);
         A0[i+16] = a0;
         a1 += w16 * z1;
         prefA(A1);
         A1[i+16] = a1;
         a2 += w16 * z2;
         prefA(A2);
         A2[i+16] = a2;
         a3 += w16 * z3;
         prefA(A3);
         A3[i+16] = a3;
         a4 += w16 * z4;
         prefA(A4);
         A4[i+16] = a4;
         a5 += w16 * z5;
         prefA(A5);
         A5[i+16] = a5;
         a6 += w16 * z6;
         prefA(A6);
         A6[i+16] = a6;
         a7 += w16 * z7;
         prefA(A7);
         A7[i+16] = a7;
         a8 += w16 * z8;
         prefA(A8);
         A8[i+16] = a8;
         a9 += w16 * z9;
         prefA(A9);
         A9[i+16] = a9;
         a10 += w16 * z10;
         prefA(A10);
         A10[i+16] = a10;
         a11 += w16 * z11;
         prefA(A11);
         A11[i+16] = a11;
         a12 += w16 * z12;
         prefA(A12);
         A12[i+16] = a12;
         a13 += w16 * z13;
         prefA(A13);
         A13[i+16] = a13;
         a14 += w16 * z14;
         prefA(A14);
         A14[i+16] = a14;
         a15 += w16 * z15;
         prefA(A15);
         A15[i+16] = a15;
         a0 = A0[i+17];
         a0 += x17 * y0;
         a1 = A1[i+17];
         a1 += x17 * y1;
         a2 = A2[i+17];
         a2 += x17 * y2;
         a3 = A3[i+17];
         a3 += x17 * y3;
         a4 = A4[i+17];
         a4 += x17 * y4;
         a5 = A5[i+17];
         a5 += x17 * y5;
         a6 = A6[i+17];
         a6 += x17 * y6;
         a7 = A7[i+17];
         a7 += x17 * y7;
         a8 = A8[i+17];
         a8 += x17 * y8;
         a9 = A9[i+17];
         a9 += x17 * y9;
         a10 = A10[i+17];
         a10 += x17 * y10;
         a11 = A11[i+17];
         a11 += x17 * y11;
         a12 = A12[i+17];
         a12 += x17 * y12;
         a13 = A13[i+17];
         a13 += x17 * y13;
         a14 = A14[i+17];
         a14 += x17 * y14;
         a15 = A15[i+17];
         a15 += x17 * y15;
         a0 += w17 * z0;
         A0[i+17] = a0;
         a1 += w17 * z1;
         A1[i+17] = a1;
         a2 += w17 * z2;
         A2[i+17] = a2;
         a3 += w17 * z3;
         A3[i+17] = a3;
         a4 += w17 * z4;
         A4[i+17] = a4;
         a5 += w17 * z5;
         A5[i+17] = a5;
         a6 += w17 * z6;
         A6[i+17] = a6;
         a7 += w17 * z7;
         A7[i+17] = a7;
         a8 += w17 * z8;
         A8[i+17] = a8;
         a9 += w17 * z9;
         A9[i+17] = a9;
         a10 += w17 * z10;
         A10[i+17] = a10;
         a11 += w17 * z11;
         A11[i+17] = a11;
         a12 += w17 * z12;
         A12[i+17] = a12;
         a13 += w17 * z13;
         A13[i+17] = a13;
         a14 += w17 * z14;
         A14[i+17] = a14;
         a15 += w17 * z15;
         A15[i+17] = a15;
         a0 = A0[i+18];
         a0 += x18 * y0;
         a1 = A1[i+18];
         a1 += x18 * y1;
         a2 = A2[i+18];
         a2 += x18 * y2;
         a3 = A3[i+18];
         a3 += x18 * y3;
         a4 = A4[i+18];
         a4 += x18 * y4;
         a5 = A5[i+18];
         a5 += x18 * y5;
         a6 = A6[i+18];
         a6 += x18 * y6;
         a7 = A7[i+18];
         a7 += x18 * y7;
         a8 = A8[i+18];
         a8 += x18 * y8;
         a9 = A9[i+18];
         a9 += x18 * y9;
         a10 = A10[i+18];
         a10 += x18 * y10;
         a11 = A11[i+18];
         a11 += x18 * y11;
         a12 = A12[i+18];
         a12 += x18 * y12;
         a13 = A13[i+18];
         a13 += x18 * y13;
         a14 = A14[i+18];
         a14 += x18 * y14;
         a15 = A15[i+18];
         a15 += x18 * y15;
         a0 += w18 * z0;
         A0[i+18] = a0;
         a1 += w18 * z1;
         A1[i+18] = a1;
         a2 += w18 * z2;
         A2[i+18] = a2;
         a3 += w18 * z3;
         A3[i+18] = a3;
         a4 += w18 * z4;
         A4[i+18] = a4;
         a5 += w18 * z5;
         A5[i+18] = a5;
         a6 += w18 * z6;
         A6[i+18] = a6;
         a7 += w18 * z7;
         A7[i+18] = a7;
         a8 += w18 * z8;
         A8[i+18] = a8;
         a9 += w18 * z9;
         A9[i+18] = a9;
         a10 += w18 * z10;
         A10[i+18] = a10;
         a11 += w18 * z11;
         A11[i+18] = a11;
         a12 += w18 * z12;
         A12[i+18] = a12;
         a13 += w18 * z13;
         A13[i+18] = a13;
         a14 += w18 * z14;
         A14[i+18] = a14;
         a15 += w18 * z15;
         A15[i+18] = a15;
         a0 = A0[i+19];
         a0 += x19 * y0;
         a1 = A1[i+19];
         a1 += x19 * y1;
         a2 = A2[i+19];
         a2 += x19 * y2;
         a3 = A3[i+19];
         a3 += x19 * y3;
         a4 = A4[i+19];
         a4 += x19 * y4;
         a5 = A5[i+19];
         a5 += x19 * y5;
         a6 = A6[i+19];
         a6 += x19 * y6;
         a7 = A7[i+19];
         a7 += x19 * y7;
         a8 = A8[i+19];
         a8 += x19 * y8;
         a9 = A9[i+19];
         a9 += x19 * y9;
         a10 = A10[i+19];
         a10 += x19 * y10;
         a11 = A11[i+19];
         a11 += x19 * y11;
         a12 = A12[i+19];
         a12 += x19 * y12;
         a13 = A13[i+19];
         a13 += x19 * y13;
         a14 = A14[i+19];
         a14 += x19 * y14;
         a15 = A15[i+19];
         a15 += x19 * y15;
         a0 += w19 * z0;
         A0[i+19] = a0;
         a1 += w19 * z1;
         A1[i+19] = a1;
         a2 += w19 * z2;
         A2[i+19] = a2;
         a3 += w19 * z3;
         A3[i+19] = a3;
         a4 += w19 * z4;
         A4[i+19] = a4;
         a5 += w19 * z5;
         A5[i+19] = a5;
         a6 += w19 * z6;
         A6[i+19] = a6;
         a7 += w19 * z7;
         A7[i+19] = a7;
         a8 += w19 * z8;
         A8[i+19] = a8;
         a9 += w19 * z9;
         A9[i+19] = a9;
         a10 += w19 * z10;
         A10[i+19] = a10;
         a11 += w19 * z11;
         A11[i+19] = a11;
         a12 += w19 * z12;
         A12[i+19] = a12;
         a13 += w19 * z13;
         A13[i+19] = a13;
         a14 += w19 * z14;
         A14[i+19] = a14;
         a15 += w19 * z15;
         A15[i+19] = a15;
         a0 = A0[i+20];
         a0 += x20 * y0;
         a1 = A1[i+20];
         a1 += x20 * y1;
         a2 = A2[i+20];
         a2 += x20 * y2;
         a3 = A3[i+20];
         a3 += x20 * y3;
         a4 = A4[i+20];
         a4 += x20 * y4;
         a5 = A5[i+20];
         a5 += x20 * y5;
         a6 = A6[i+20];
         a6 += x20 * y6;
         a7 = A7[i+20];
         a7 += x20 * y7;
         a8 = A8[i+20];
         a8 += x20 * y8;
         a9 = A9[i+20];
         a9 += x20 * y9;
         a10 = A10[i+20];
         a10 += x20 * y10;
         a11 = A11[i+20];
         a11 += x20 * y11;
         a12 = A12[i+20];
         a12 += x20 * y12;
         a13 = A13[i+20];
         a13 += x20 * y13;
         a14 = A14[i+20];
         a14 += x20 * y14;
         a15 = A15[i+20];
         a15 += x20 * y15;
         a0 += w20 * z0;
         A0[i+20] = a0;
         a1 += w20 * z1;
         A1[i+20] = a1;
         a2 += w20 * z2;
         A2[i+20] = a2;
         a3 += w20 * z3;
         A3[i+20] = a3;
         a4 += w20 * z4;
         A4[i+20] = a4;
         a5 += w20 * z5;
         A5[i+20] = a5;
         a6 += w20 * z6;
         A6[i+20] = a6;
         a7 += w20 * z7;
         A7[i+20] = a7;
         a8 += w20 * z8;
         A8[i+20] = a8;
         a9 += w20 * z9;
         A9[i+20] = a9;
         a10 += w20 * z10;
         A10[i+20] = a10;
         a11 += w20 * z11;
         A11[i+20] = a11;
         a12 += w20 * z12;
         A12[i+20] = a12;
         a13 += w20 * z13;
         A13[i+20] = a13;
         a14 += w20 * z14;
         A14[i+20] = a14;
         a15 += w20 * z15;
         A15[i+20] = a15;
         a0 = A0[i+21];
         a0 += x21 * y0;
         a1 = A1[i+21];
         a1 += x21 * y1;
         a2 = A2[i+21];
         a2 += x21 * y2;
         a3 = A3[i+21];
         a3 += x21 * y3;
         a4 = A4[i+21];
         a4 += x21 * y4;
         a5 = A5[i+21];
         a5 += x21 * y5;
         a6 = A6[i+21];
         a6 += x21 * y6;
         a7 = A7[i+21];
         a7 += x21 * y7;
         a8 = A8[i+21];
         a8 += x21 * y8;
         a9 = A9[i+21];
         a9 += x21 * y9;
         a10 = A10[i+21];
         a10 += x21 * y10;
         a11 = A11[i+21];
         a11 += x21 * y11;
         a12 = A12[i+21];
         a12 += x21 * y12;
         a13 = A13[i+21];
         a13 += x21 * y13;
         a14 = A14[i+21];
         a14 += x21 * y14;
         a15 = A15[i+21];
         a15 += x21 * y15;
         a0 += w21 * z0;
         A0[i+21] = a0;
         a1 += w21 * z1;
         A1[i+21] = a1;
         a2 += w21 * z2;
         A2[i+21] = a2;
         a3 += w21 * z3;
         A3[i+21] = a3;
         a4 += w21 * z4;
         A4[i+21] = a4;
         a5 += w21 * z5;
         A5[i+21] = a5;
         a6 += w21 * z6;
         A6[i+21] = a6;
         a7 += w21 * z7;
         A7[i+21] = a7;
         a8 += w21 * z8;
         A8[i+21] = a8;
         a9 += w21 * z9;
         A9[i+21] = a9;
         a10 += w21 * z10;
         A10[i+21] = a10;
         a11 += w21 * z11;
         A11[i+21] = a11;
         a12 += w21 * z12;
         A12[i+21] = a12;
         a13 += w21 * z13;
         A13[i+21] = a13;
         a14 += w21 * z14;
         A14[i+21] = a14;
         a15 += w21 * z15;
         A15[i+21] = a15;
         a0 = A0[i+22];
         a0 += x22 * y0;
         a1 = A1[i+22];
         a1 += x22 * y1;
         a2 = A2[i+22];
         a2 += x22 * y2;
         a3 = A3[i+22];
         a3 += x22 * y3;
         a4 = A4[i+22];
         a4 += x22 * y4;
         a5 = A5[i+22];
         a5 += x22 * y5;
         a6 = A6[i+22];
         a6 += x22 * y6;
         a7 = A7[i+22];
         a7 += x22 * y7;
         a8 = A8[i+22];
         a8 += x22 * y8;
         a9 = A9[i+22];
         a9 += x22 * y9;
         a10 = A10[i+22];
         a10 += x22 * y10;
         a11 = A11[i+22];
         a11 += x22 * y11;
         a12 = A12[i+22];
         a12 += x22 * y12;
         a13 = A13[i+22];
         a13 += x22 * y13;
         a14 = A14[i+22];
         a14 += x22 * y14;
         a15 = A15[i+22];
         a15 += x22 * y15;
         a0 += w22 * z0;
         A0[i+22] = a0;
         a1 += w22 * z1;
         A1[i+22] = a1;
         a2 += w22 * z2;
         A2[i+22] = a2;
         a3 += w22 * z3;
         A3[i+22] = a3;
         a4 += w22 * z4;
         A4[i+22] = a4;
         a5 += w22 * z5;
         A5[i+22] = a5;
         a6 += w22 * z6;
         A6[i+22] = a6;
         a7 += w22 * z7;
         A7[i+22] = a7;
         a8 += w22 * z8;
         A8[i+22] = a8;
         a9 += w22 * z9;
         A9[i+22] = a9;
         a10 += w22 * z10;
         A10[i+22] = a10;
         a11 += w22 * z11;
         A11[i+22] = a11;
         a12 += w22 * z12;
         A12[i+22] = a12;
         a13 += w22 * z13;
         A13[i+22] = a13;
         a14 += w22 * z14;
         A14[i+22] = a14;
         a15 += w22 * z15;
         A15[i+22] = a15;
         a0 = A0[i+23];
         a0 += x23 * y0;
         a1 = A1[i+23];
         a1 += x23 * y1;
         a2 = A2[i+23];
         a2 += x23 * y2;
         a3 = A3[i+23];
         a3 += x23 * y3;
         a4 = A4[i+23];
         a4 += x23 * y4;
         a5 = A5[i+23];
         a5 += x23 * y5;
         a6 = A6[i+23];
         a6 += x23 * y6;
         a7 = A7[i+23];
         a7 += x23 * y7;
         a8 = A8[i+23];
         a8 += x23 * y8;
         a9 = A9[i+23];
         a9 += x23 * y9;
         a10 = A10[i+23];
         a10 += x23 * y10;
         a11 = A11[i+23];
         a11 += x23 * y11;
         a12 = A12[i+23];
         a12 += x23 * y12;
         a13 = A13[i+23];
         a13 += x23 * y13;
         a14 = A14[i+23];
         a14 += x23 * y14;
         a15 = A15[i+23];
         a15 += x23 * y15;
         a0 += w23 * z0;
         A0[i+23] = a0;
         a1 += w23 * z1;
         A1[i+23] = a1;
         a2 += w23 * z2;
         A2[i+23] = a2;
         a3 += w23 * z3;
         A3[i+23] = a3;
         a4 += w23 * z4;
         A4[i+23] = a4;
         a5 += w23 * z5;
         A5[i+23] = a5;
         a6 += w23 * z6;
         A6[i+23] = a6;
         a7 += w23 * z7;
         A7[i+23] = a7;
         a8 += w23 * z8;
         A8[i+23] = a8;
         a9 += w23 * z9;
         A9[i+23] = a9;
         a10 += w23 * z10;
         A10[i+23] = a10;
         a11 += w23 * z11;
         A11[i+23] = a11;
         a12 += w23 * z12;
         A12[i+23] = a12;
         a13 += w23 * z13;
         A13[i+23] = a13;
         a14 += w23 * z14;
         A14[i+23] = a14;
         a15 += w23 * z15;
         A15[i+23] = a15;
         a0 = A0[i+24];
         a0 += x24 * y0;
         a1 = A1[i+24];
         a1 += x24 * y1;
         a2 = A2[i+24];
         a2 += x24 * y2;
         a3 = A3[i+24];
         a3 += x24 * y3;
         a4 = A4[i+24];
         a4 += x24 * y4;
         a5 = A5[i+24];
         a5 += x24 * y5;
         a6 = A6[i+24];
         a6 += x24 * y6;
         a7 = A7[i+24];
         a7 += x24 * y7;
         a8 = A8[i+24];
         a8 += x24 * y8;
         a9 = A9[i+24];
         a9 += x24 * y9;
         a10 = A10[i+24];
         a10 += x24 * y10;
         a11 = A11[i+24];
         a11 += x24 * y11;
         a12 = A12[i+24];
         a12 += x24 * y12;
         a13 = A13[i+24];
         a13 += x24 * y13;
         a14 = A14[i+24];
         a14 += x24 * y14;
         a15 = A15[i+24];
         a15 += x24 * y15;
         a0 += w24 * z0;
         A0[i+24] = a0;
         a1 += w24 * z1;
         A1[i+24] = a1;
         a2 += w24 * z2;
         A2[i+24] = a2;
         a3 += w24 * z3;
         A3[i+24] = a3;
         a4 += w24 * z4;
         A4[i+24] = a4;
         a5 += w24 * z5;
         A5[i+24] = a5;
         a6 += w24 * z6;
         A6[i+24] = a6;
         a7 += w24 * z7;
         A7[i+24] = a7;
         a8 += w24 * z8;
         A8[i+24] = a8;
         a9 += w24 * z9;
         A9[i+24] = a9;
         a10 += w24 * z10;
         A10[i+24] = a10;
         a11 += w24 * z11;
         A11[i+24] = a11;
         a12 += w24 * z12;
         A12[i+24] = a12;
         a13 += w24 * z13;
         A13[i+24] = a13;
         a14 += w24 * z14;
         A14[i+24] = a14;
         a15 += w24 * z15;
         A15[i+24] = a15;
         a0 = A0[i+25];
         a0 += x25 * y0;
         a1 = A1[i+25];
         a1 += x25 * y1;
         a2 = A2[i+25];
         a2 += x25 * y2;
         a3 = A3[i+25];
         a3 += x25 * y3;
         a4 = A4[i+25];
         a4 += x25 * y4;
         a5 = A5[i+25];
         a5 += x25 * y5;
         a6 = A6[i+25];
         a6 += x25 * y6;
         a7 = A7[i+25];
         a7 += x25 * y7;
         a8 = A8[i+25];
         a8 += x25 * y8;
         a9 = A9[i+25];
         a9 += x25 * y9;
         a10 = A10[i+25];
         a10 += x25 * y10;
         a11 = A11[i+25];
         a11 += x25 * y11;
         a12 = A12[i+25];
         a12 += x25 * y12;
         a13 = A13[i+25];
         a13 += x25 * y13;
         a14 = A14[i+25];
         a14 += x25 * y14;
         a15 = A15[i+25];
         a15 += x25 * y15;
         a0 += w25 * z0;
         A0[i+25] = a0;
         a1 += w25 * z1;
         A1[i+25] = a1;
         a2 += w25 * z2;
         A2[i+25] = a2;
         a3 += w25 * z3;
         A3[i+25] = a3;
         a4 += w25 * z4;
         A4[i+25] = a4;
         a5 += w25 * z5;
         A5[i+25] = a5;
         a6 += w25 * z6;
         A6[i+25] = a6;
         a7 += w25 * z7;
         A7[i+25] = a7;
         a8 += w25 * z8;
         A8[i+25] = a8;
         a9 += w25 * z9;
         A9[i+25] = a9;
         a10 += w25 * z10;
         A10[i+25] = a10;
         a11 += w25 * z11;
         A11[i+25] = a11;
         a12 += w25 * z12;
         A12[i+25] = a12;
         a13 += w25 * z13;
         A13[i+25] = a13;
         a14 += w25 * z14;
         A14[i+25] = a14;
         a15 += w25 * z15;
         A15[i+25] = a15;
         a0 = A0[i+26];
         a0 += x26 * y0;
         a1 = A1[i+26];
         a1 += x26 * y1;
         a2 = A2[i+26];
         a2 += x26 * y2;
         a3 = A3[i+26];
         a3 += x26 * y3;
         a4 = A4[i+26];
         a4 += x26 * y4;
         a5 = A5[i+26];
         a5 += x26 * y5;
         a6 = A6[i+26];
         a6 += x26 * y6;
         a7 = A7[i+26];
         a7 += x26 * y7;
         a8 = A8[i+26];
         a8 += x26 * y8;
         a9 = A9[i+26];
         a9 += x26 * y9;
         a10 = A10[i+26];
         a10 += x26 * y10;
         a11 = A11[i+26];
         a11 += x26 * y11;
         a12 = A12[i+26];
         a12 += x26 * y12;
         a13 = A13[i+26];
         a13 += x26 * y13;
         a14 = A14[i+26];
         a14 += x26 * y14;
         a15 = A15[i+26];
         a15 += x26 * y15;
         a0 += w26 * z0;
         A0[i+26] = a0;
         a1 += w26 * z1;
         A1[i+26] = a1;
         a2 += w26 * z2;
         A2[i+26] = a2;
         a3 += w26 * z3;
         A3[i+26] = a3;
         a4 += w26 * z4;
         A4[i+26] = a4;
         a5 += w26 * z5;
         A5[i+26] = a5;
         a6 += w26 * z6;
         A6[i+26] = a6;
         a7 += w26 * z7;
         A7[i+26] = a7;
         a8 += w26 * z8;
         A8[i+26] = a8;
         a9 += w26 * z9;
         A9[i+26] = a9;
         a10 += w26 * z10;
         A10[i+26] = a10;
         a11 += w26 * z11;
         A11[i+26] = a11;
         a12 += w26 * z12;
         A12[i+26] = a12;
         a13 += w26 * z13;
         A13[i+26] = a13;
         a14 += w26 * z14;
         A14[i+26] = a14;
         a15 += w26 * z15;
         A15[i+26] = a15;
         a0 = A0[i+27];
         a0 += x27 * y0;
         a1 = A1[i+27];
         a1 += x27 * y1;
         a2 = A2[i+27];
         a2 += x27 * y2;
         a3 = A3[i+27];
         a3 += x27 * y3;
         a4 = A4[i+27];
         a4 += x27 * y4;
         a5 = A5[i+27];
         a5 += x27 * y5;
         a6 = A6[i+27];
         a6 += x27 * y6;
         a7 = A7[i+27];
         a7 += x27 * y7;
         a8 = A8[i+27];
         a8 += x27 * y8;
         a9 = A9[i+27];
         a9 += x27 * y9;
         a10 = A10[i+27];
         a10 += x27 * y10;
         a11 = A11[i+27];
         a11 += x27 * y11;
         a12 = A12[i+27];
         a12 += x27 * y12;
         a13 = A13[i+27];
         a13 += x27 * y13;
         a14 = A14[i+27];
         a14 += x27 * y14;
         a15 = A15[i+27];
         a15 += x27 * y15;
         a0 += w27 * z0;
         A0[i+27] = a0;
         a1 += w27 * z1;
         A1[i+27] = a1;
         a2 += w27 * z2;
         A2[i+27] = a2;
         a3 += w27 * z3;
         A3[i+27] = a3;
         a4 += w27 * z4;
         A4[i+27] = a4;
         a5 += w27 * z5;
         A5[i+27] = a5;
         a6 += w27 * z6;
         A6[i+27] = a6;
         a7 += w27 * z7;
         A7[i+27] = a7;
         a8 += w27 * z8;
         A8[i+27] = a8;
         a9 += w27 * z9;
         A9[i+27] = a9;
         a10 += w27 * z10;
         A10[i+27] = a10;
         a11 += w27 * z11;
         A11[i+27] = a11;
         a12 += w27 * z12;
         A12[i+27] = a12;
         a13 += w27 * z13;
         A13[i+27] = a13;
         a14 += w27 * z14;
         A14[i+27] = a14;
         a15 += w27 * z15;
         A15[i+27] = a15;
         a0 = A0[i+28];
         a0 += x28 * y0;
         a1 = A1[i+28];
         a1 += x28 * y1;
         a2 = A2[i+28];
         a2 += x28 * y2;
         a3 = A3[i+28];
         a3 += x28 * y3;
         a4 = A4[i+28];
         a4 += x28 * y4;
         a5 = A5[i+28];
         a5 += x28 * y5;
         a6 = A6[i+28];
         a6 += x28 * y6;
         a7 = A7[i+28];
         a7 += x28 * y7;
         a8 = A8[i+28];
         a8 += x28 * y8;
         a9 = A9[i+28];
         a9 += x28 * y9;
         a10 = A10[i+28];
         a10 += x28 * y10;
         a11 = A11[i+28];
         a11 += x28 * y11;
         a12 = A12[i+28];
         a12 += x28 * y12;
         a13 = A13[i+28];
         a13 += x28 * y13;
         a14 = A14[i+28];
         a14 += x28 * y14;
         a15 = A15[i+28];
         a15 += x28 * y15;
         a0 += w28 * z0;
         A0[i+28] = a0;
         a1 += w28 * z1;
         A1[i+28] = a1;
         a2 += w28 * z2;
         A2[i+28] = a2;
         a3 += w28 * z3;
         A3[i+28] = a3;
         a4 += w28 * z4;
         A4[i+28] = a4;
         a5 += w28 * z5;
         A5[i+28] = a5;
         a6 += w28 * z6;
         A6[i+28] = a6;
         a7 += w28 * z7;
         A7[i+28] = a7;
         a8 += w28 * z8;
         A8[i+28] = a8;
         a9 += w28 * z9;
         A9[i+28] = a9;
         a10 += w28 * z10;
         A10[i+28] = a10;
         a11 += w28 * z11;
         A11[i+28] = a11;
         a12 += w28 * z12;
         A12[i+28] = a12;
         a13 += w28 * z13;
         A13[i+28] = a13;
         a14 += w28 * z14;
         A14[i+28] = a14;
         a15 += w28 * z15;
         A15[i+28] = a15;
         a0 = A0[i+29];
         a0 += x29 * y0;
         a1 = A1[i+29];
         a1 += x29 * y1;
         a2 = A2[i+29];
         a2 += x29 * y2;
         a3 = A3[i+29];
         a3 += x29 * y3;
         a4 = A4[i+29];
         a4 += x29 * y4;
         a5 = A5[i+29];
         a5 += x29 * y5;
         a6 = A6[i+29];
         a6 += x29 * y6;
         a7 = A7[i+29];
         a7 += x29 * y7;
         a8 = A8[i+29];
         a8 += x29 * y8;
         a9 = A9[i+29];
         a9 += x29 * y9;
         a10 = A10[i+29];
         a10 += x29 * y10;
         a11 = A11[i+29];
         a11 += x29 * y11;
         a12 = A12[i+29];
         a12 += x29 * y12;
         a13 = A13[i+29];
         a13 += x29 * y13;
         a14 = A14[i+29];
         a14 += x29 * y14;
         a15 = A15[i+29];
         a15 += x29 * y15;
         a0 += w29 * z0;
         A0[i+29] = a0;
         a1 += w29 * z1;
         A1[i+29] = a1;
         a2 += w29 * z2;
         A2[i+29] = a2;
         a3 += w29 * z3;
         A3[i+29] = a3;
         a4 += w29 * z4;
         A4[i+29] = a4;
         a5 += w29 * z5;
         A5[i+29] = a5;
         a6 += w29 * z6;
         A6[i+29] = a6;
         a7 += w29 * z7;
         A7[i+29] = a7;
         a8 += w29 * z8;
         A8[i+29] = a8;
         a9 += w29 * z9;
         A9[i+29] = a9;
         a10 += w29 * z10;
         A10[i+29] = a10;
         a11 += w29 * z11;
         A11[i+29] = a11;
         a12 += w29 * z12;
         A12[i+29] = a12;
         a13 += w29 * z13;
         A13[i+29] = a13;
         a14 += w29 * z14;
         A14[i+29] = a14;
         a15 += w29 * z15;
         A15[i+29] = a15;
         a0 = A0[i+30];
         a0 += x30 * y0;
         a1 = A1[i+30];
         a1 += x30 * y1;
         a2 = A2[i+30];
         a2 += x30 * y2;
         a3 = A3[i+30];
         a3 += x30 * y3;
         a4 = A4[i+30];
         a4 += x30 * y4;
         a5 = A5[i+30];
         a5 += x30 * y5;
         a6 = A6[i+30];
         a6 += x30 * y6;
         a7 = A7[i+30];
         a7 += x30 * y7;
         a8 = A8[i+30];
         a8 += x30 * y8;
         a9 = A9[i+30];
         a9 += x30 * y9;
         a10 = A10[i+30];
         a10 += x30 * y10;
         a11 = A11[i+30];
         a11 += x30 * y11;
         a12 = A12[i+30];
         a12 += x30 * y12;
         a13 = A13[i+30];
         a13 += x30 * y13;
         a14 = A14[i+30];
         a14 += x30 * y14;
         a15 = A15[i+30];
         a15 += x30 * y15;
         a0 += w30 * z0;
         A0[i+30] = a0;
         a1 += w30 * z1;
         A1[i+30] = a1;
         a2 += w30 * z2;
         A2[i+30] = a2;
         a3 += w30 * z3;
         A3[i+30] = a3;
         a4 += w30 * z4;
         A4[i+30] = a4;
         a5 += w30 * z5;
         A5[i+30] = a5;
         a6 += w30 * z6;
         A6[i+30] = a6;
         a7 += w30 * z7;
         A7[i+30] = a7;
         a8 += w30 * z8;
         A8[i+30] = a8;
         a9 += w30 * z9;
         A9[i+30] = a9;
         a10 += w30 * z10;
         A10[i+30] = a10;
         a11 += w30 * z11;
         A11[i+30] = a11;
         a12 += w30 * z12;
         A12[i+30] = a12;
         a13 += w30 * z13;
         A13[i+30] = a13;
         a14 += w30 * z14;
         A14[i+30] = a14;
         a15 += w30 * z15;
         A15[i+30] = a15;
         a0 = A0[i+31];
         a0 += x31 * y0;
         a1 = A1[i+31];
         a1 += x31 * y1;
         a2 = A2[i+31];
         a2 += x31 * y2;
         a3 = A3[i+31];
         a3 += x31 * y3;
         a4 = A4[i+31];
         a4 += x31 * y4;
         a5 = A5[i+31];
         a5 += x31 * y5;
         a6 = A6[i+31];
         a6 += x31 * y6;
         a7 = A7[i+31];
         a7 += x31 * y7;
         a8 = A8[i+31];
         a8 += x31 * y8;
         a9 = A9[i+31];
         a9 += x31 * y9;
         a10 = A10[i+31];
         a10 += x31 * y10;
         a11 = A11[i+31];
         a11 += x31 * y11;
         a12 = A12[i+31];
         a12 += x31 * y12;
         a13 = A13[i+31];
         a13 += x31 * y13;
         a14 = A14[i+31];
         a14 += x31 * y14;
         a15 = A15[i+31];
         a15 += x31 * y15;
         a0 += w31 * z0;
         A0[i+31] = a0;
         a1 += w31 * z1;
         A1[i+31] = a1;
         a2 += w31 * z2;
         A2[i+31] = a2;
         a3 += w31 * z3;
         A3[i+31] = a3;
         a4 += w31 * z4;
         A4[i+31] = a4;
         a5 += w31 * z5;
         A5[i+31] = a5;
         a6 += w31 * z6;
         A6[i+31] = a6;
         a7 += w31 * z7;
         A7[i+31] = a7;
         a8 += w31 * z8;
         A8[i+31] = a8;
         a9 += w31 * z9;
         A9[i+31] = a9;
         a10 += w31 * z10;
         A10[i+31] = a10;
         a11 += w31 * z11;
         A11[i+31] = a11;
         a12 += w31 * z12;
         A12[i+31] = a12;
         a13 += w31 * z13;
         A13[i+31] = a13;
         a14 += w31 * z14;
         A14[i+31] = a14;
         a15 += w31 * z15;
         A15[i+31] = a15;
         a0 = A0[i+32];
         a0 += x32 * y0;
         a1 = A1[i+32];
         a1 += x32 * y1;
         a2 = A2[i+32];
         a2 += x32 * y2;
         a3 = A3[i+32];
         a3 += x32 * y3;
         a4 = A4[i+32];
         a4 += x32 * y4;
         a5 = A5[i+32];
         a5 += x32 * y5;
         a6 = A6[i+32];
         a6 += x32 * y6;
         a7 = A7[i+32];
         a7 += x32 * y7;
         a8 = A8[i+32];
         a8 += x32 * y8;
         a9 = A9[i+32];
         a9 += x32 * y9;
         a10 = A10[i+32];
         a10 += x32 * y10;
         a11 = A11[i+32];
         a11 += x32 * y11;
         a12 = A12[i+32];
         a12 += x32 * y12;
         a13 = A13[i+32];
         a13 += x32 * y13;
         a14 = A14[i+32];
         a14 += x32 * y14;
         a15 = A15[i+32];
         a15 += x32 * y15;
         a0 += w32 * z0;
         prefA(A0);
         A0[i+32] = a0;
         a1 += w32 * z1;
         prefA(A1);
         A1[i+32] = a1;
         a2 += w32 * z2;
         prefA(A2);
         A2[i+32] = a2;
         a3 += w32 * z3;
         prefA(A3);
         A3[i+32] = a3;
         a4 += w32 * z4;
         prefA(A4);
         A4[i+32] = a4;
         a5 += w32 * z5;
         prefA(A5);
         A5[i+32] = a5;
         a6 += w32 * z6;
         prefA(A6);
         A6[i+32] = a6;
         a7 += w32 * z7;
         prefA(A7);
         A7[i+32] = a7;
         a8 += w32 * z8;
         prefA(A8);
         A8[i+32] = a8;
         a9 += w32 * z9;
         prefA(A9);
         A9[i+32] = a9;
         a10 += w32 * z10;
         prefA(A10);
         A10[i+32] = a10;
         a11 += w32 * z11;
         prefA(A11);
         A11[i+32] = a11;
         a12 += w32 * z12;
         prefA(A12);
         A12[i+32] = a12;
         a13 += w32 * z13;
         prefA(A13);
         A13[i+32] = a13;
         a14 += w32 * z14;
         prefA(A14);
         A14[i+32] = a14;
         a15 += w32 * z15;
         prefA(A15);
         A15[i+32] = a15;
         a0 = A0[i+33];
         a0 += x33 * y0;
         a1 = A1[i+33];
         a1 += x33 * y1;
         a2 = A2[i+33];
         a2 += x33 * y2;
         a3 = A3[i+33];
         a3 += x33 * y3;
         a4 = A4[i+33];
         a4 += x33 * y4;
         a5 = A5[i+33];
         a5 += x33 * y5;
         a6 = A6[i+33];
         a6 += x33 * y6;
         a7 = A7[i+33];
         a7 += x33 * y7;
         a8 = A8[i+33];
         a8 += x33 * y8;
         a9 = A9[i+33];
         a9 += x33 * y9;
         a10 = A10[i+33];
         a10 += x33 * y10;
         a11 = A11[i+33];
         a11 += x33 * y11;
         a12 = A12[i+33];
         a12 += x33 * y12;
         a13 = A13[i+33];
         a13 += x33 * y13;
         a14 = A14[i+33];
         a14 += x33 * y14;
         a15 = A15[i+33];
         a15 += x33 * y15;
         a0 += w33 * z0;
         A0[i+33] = a0;
         a1 += w33 * z1;
         A1[i+33] = a1;
         a2 += w33 * z2;
         A2[i+33] = a2;
         a3 += w33 * z3;
         A3[i+33] = a3;
         a4 += w33 * z4;
         A4[i+33] = a4;
         a5 += w33 * z5;
         A5[i+33] = a5;
         a6 += w33 * z6;
         A6[i+33] = a6;
         a7 += w33 * z7;
         A7[i+33] = a7;
         a8 += w33 * z8;
         A8[i+33] = a8;
         a9 += w33 * z9;
         A9[i+33] = a9;
         a10 += w33 * z10;
         A10[i+33] = a10;
         a11 += w33 * z11;
         A11[i+33] = a11;
         a12 += w33 * z12;
         A12[i+33] = a12;
         a13 += w33 * z13;
         A13[i+33] = a13;
         a14 += w33 * z14;
         A14[i+33] = a14;
         a15 += w33 * z15;
         A15[i+33] = a15;
         a0 = A0[i+34];
         a0 += x34 * y0;
         a1 = A1[i+34];
         a1 += x34 * y1;
         a2 = A2[i+34];
         a2 += x34 * y2;
         a3 = A3[i+34];
         a3 += x34 * y3;
         a4 = A4[i+34];
         a4 += x34 * y4;
         a5 = A5[i+34];
         a5 += x34 * y5;
         a6 = A6[i+34];
         a6 += x34 * y6;
         a7 = A7[i+34];
         a7 += x34 * y7;
         a8 = A8[i+34];
         a8 += x34 * y8;
         a9 = A9[i+34];
         a9 += x34 * y9;
         a10 = A10[i+34];
         a10 += x34 * y10;
         a11 = A11[i+34];
         a11 += x34 * y11;
         a12 = A12[i+34];
         a12 += x34 * y12;
         a13 = A13[i+34];
         a13 += x34 * y13;
         a14 = A14[i+34];
         a14 += x34 * y14;
         a15 = A15[i+34];
         a15 += x34 * y15;
         a0 += w34 * z0;
         A0[i+34] = a0;
         a1 += w34 * z1;
         A1[i+34] = a1;
         a2 += w34 * z2;
         A2[i+34] = a2;
         a3 += w34 * z3;
         A3[i+34] = a3;
         a4 += w34 * z4;
         A4[i+34] = a4;
         a5 += w34 * z5;
         A5[i+34] = a5;
         a6 += w34 * z6;
         A6[i+34] = a6;
         a7 += w34 * z7;
         A7[i+34] = a7;
         a8 += w34 * z8;
         A8[i+34] = a8;
         a9 += w34 * z9;
         A9[i+34] = a9;
         a10 += w34 * z10;
         A10[i+34] = a10;
         a11 += w34 * z11;
         A11[i+34] = a11;
         a12 += w34 * z12;
         A12[i+34] = a12;
         a13 += w34 * z13;
         A13[i+34] = a13;
         a14 += w34 * z14;
         A14[i+34] = a14;
         a15 += w34 * z15;
         A15[i+34] = a15;
         a0 = A0[i+35];
         a0 += x35 * y0;
         a1 = A1[i+35];
         a1 += x35 * y1;
         a2 = A2[i+35];
         a2 += x35 * y2;
         a3 = A3[i+35];
         a3 += x35 * y3;
         a4 = A4[i+35];
         a4 += x35 * y4;
         a5 = A5[i+35];
         a5 += x35 * y5;
         a6 = A6[i+35];
         a6 += x35 * y6;
         a7 = A7[i+35];
         a7 += x35 * y7;
         a8 = A8[i+35];
         a8 += x35 * y8;
         a9 = A9[i+35];
         a9 += x35 * y9;
         a10 = A10[i+35];
         a10 += x35 * y10;
         a11 = A11[i+35];
         a11 += x35 * y11;
         a12 = A12[i+35];
         a12 += x35 * y12;
         a13 = A13[i+35];
         a13 += x35 * y13;
         a14 = A14[i+35];
         a14 += x35 * y14;
         a15 = A15[i+35];
         a15 += x35 * y15;
         a0 += w35 * z0;
         A0[i+35] = a0;
         a1 += w35 * z1;
         A1[i+35] = a1;
         a2 += w35 * z2;
         A2[i+35] = a2;
         a3 += w35 * z3;
         A3[i+35] = a3;
         a4 += w35 * z4;
         A4[i+35] = a4;
         a5 += w35 * z5;
         A5[i+35] = a5;
         a6 += w35 * z6;
         A6[i+35] = a6;
         a7 += w35 * z7;
         A7[i+35] = a7;
         a8 += w35 * z8;
         A8[i+35] = a8;
         a9 += w35 * z9;
         A9[i+35] = a9;
         a10 += w35 * z10;
         A10[i+35] = a10;
         a11 += w35 * z11;
         A11[i+35] = a11;
         a12 += w35 * z12;
         A12[i+35] = a12;
         a13 += w35 * z13;
         A13[i+35] = a13;
         a14 += w35 * z14;
         A14[i+35] = a14;
         a15 += w35 * z15;
         A15[i+35] = a15;
         a0 = A0[i+36];
         a0 += x36 * y0;
         a1 = A1[i+36];
         a1 += x36 * y1;
         a2 = A2[i+36];
         a2 += x36 * y2;
         a3 = A3[i+36];
         a3 += x36 * y3;
         a4 = A4[i+36];
         a4 += x36 * y4;
         a5 = A5[i+36];
         a5 += x36 * y5;
         a6 = A6[i+36];
         a6 += x36 * y6;
         a7 = A7[i+36];
         a7 += x36 * y7;
         a8 = A8[i+36];
         a8 += x36 * y8;
         a9 = A9[i+36];
         a9 += x36 * y9;
         a10 = A10[i+36];
         a10 += x36 * y10;
         a11 = A11[i+36];
         a11 += x36 * y11;
         a12 = A12[i+36];
         a12 += x36 * y12;
         a13 = A13[i+36];
         a13 += x36 * y13;
         a14 = A14[i+36];
         a14 += x36 * y14;
         a15 = A15[i+36];
         a15 += x36 * y15;
         a0 += w36 * z0;
         A0[i+36] = a0;
         a1 += w36 * z1;
         A1[i+36] = a1;
         a2 += w36 * z2;
         A2[i+36] = a2;
         a3 += w36 * z3;
         A3[i+36] = a3;
         a4 += w36 * z4;
         A4[i+36] = a4;
         a5 += w36 * z5;
         A5[i+36] = a5;
         a6 += w36 * z6;
         A6[i+36] = a6;
         a7 += w36 * z7;
         A7[i+36] = a7;
         a8 += w36 * z8;
         A8[i+36] = a8;
         a9 += w36 * z9;
         A9[i+36] = a9;
         a10 += w36 * z10;
         A10[i+36] = a10;
         a11 += w36 * z11;
         A11[i+36] = a11;
         a12 += w36 * z12;
         A12[i+36] = a12;
         a13 += w36 * z13;
         A13[i+36] = a13;
         a14 += w36 * z14;
         A14[i+36] = a14;
         a15 += w36 * z15;
         A15[i+36] = a15;
         a0 = A0[i+37];
         a0 += x37 * y0;
         a1 = A1[i+37];
         a1 += x37 * y1;
         a2 = A2[i+37];
         a2 += x37 * y2;
         a3 = A3[i+37];
         a3 += x37 * y3;
         a4 = A4[i+37];
         a4 += x37 * y4;
         a5 = A5[i+37];
         a5 += x37 * y5;
         a6 = A6[i+37];
         a6 += x37 * y6;
         a7 = A7[i+37];
         a7 += x37 * y7;
         a8 = A8[i+37];
         a8 += x37 * y8;
         a9 = A9[i+37];
         a9 += x37 * y9;
         a10 = A10[i+37];
         a10 += x37 * y10;
         a11 = A11[i+37];
         a11 += x37 * y11;
         a12 = A12[i+37];
         a12 += x37 * y12;
         a13 = A13[i+37];
         a13 += x37 * y13;
         a14 = A14[i+37];
         a14 += x37 * y14;
         a15 = A15[i+37];
         a15 += x37 * y15;
         a0 += w37 * z0;
         A0[i+37] = a0;
         a1 += w37 * z1;
         A1[i+37] = a1;
         a2 += w37 * z2;
         A2[i+37] = a2;
         a3 += w37 * z3;
         A3[i+37] = a3;
         a4 += w37 * z4;
         A4[i+37] = a4;
         a5 += w37 * z5;
         A5[i+37] = a5;
         a6 += w37 * z6;
         A6[i+37] = a6;
         a7 += w37 * z7;
         A7[i+37] = a7;
         a8 += w37 * z8;
         A8[i+37] = a8;
         a9 += w37 * z9;
         A9[i+37] = a9;
         a10 += w37 * z10;
         A10[i+37] = a10;
         a11 += w37 * z11;
         A11[i+37] = a11;
         a12 += w37 * z12;
         A12[i+37] = a12;
         a13 += w37 * z13;
         A13[i+37] = a13;
         a14 += w37 * z14;
         A14[i+37] = a14;
         a15 += w37 * z15;
         A15[i+37] = a15;
         a0 = A0[i+38];
         a0 += x38 * y0;
         a1 = A1[i+38];
         a1 += x38 * y1;
         a2 = A2[i+38];
         a2 += x38 * y2;
         a3 = A3[i+38];
         a3 += x38 * y3;
         a4 = A4[i+38];
         a4 += x38 * y4;
         a5 = A5[i+38];
         a5 += x38 * y5;
         a6 = A6[i+38];
         a6 += x38 * y6;
         a7 = A7[i+38];
         a7 += x38 * y7;
         a8 = A8[i+38];
         a8 += x38 * y8;
         a9 = A9[i+38];
         a9 += x38 * y9;
         a10 = A10[i+38];
         a10 += x38 * y10;
         a11 = A11[i+38];
         a11 += x38 * y11;
         a12 = A12[i+38];
         a12 += x38 * y12;
         a13 = A13[i+38];
         a13 += x38 * y13;
         a14 = A14[i+38];
         a14 += x38 * y14;
         a15 = A15[i+38];
         a15 += x38 * y15;
         a0 += w38 * z0;
         A0[i+38] = a0;
         a1 += w38 * z1;
         A1[i+38] = a1;
         a2 += w38 * z2;
         A2[i+38] = a2;
         a3 += w38 * z3;
         A3[i+38] = a3;
         a4 += w38 * z4;
         A4[i+38] = a4;
         a5 += w38 * z5;
         A5[i+38] = a5;
         a6 += w38 * z6;
         A6[i+38] = a6;
         a7 += w38 * z7;
         A7[i+38] = a7;
         a8 += w38 * z8;
         A8[i+38] = a8;
         a9 += w38 * z9;
         A9[i+38] = a9;
         a10 += w38 * z10;
         A10[i+38] = a10;
         a11 += w38 * z11;
         A11[i+38] = a11;
         a12 += w38 * z12;
         A12[i+38] = a12;
         a13 += w38 * z13;
         A13[i+38] = a13;
         a14 += w38 * z14;
         A14[i+38] = a14;
         a15 += w38 * z15;
         A15[i+38] = a15;
         a0 = A0[i+39];
         a0 += x39 * y0;
         a1 = A1[i+39];
         a1 += x39 * y1;
         a2 = A2[i+39];
         a2 += x39 * y2;
         a3 = A3[i+39];
         a3 += x39 * y3;
         a4 = A4[i+39];
         a4 += x39 * y4;
         a5 = A5[i+39];
         a5 += x39 * y5;
         a6 = A6[i+39];
         a6 += x39 * y6;
         a7 = A7[i+39];
         a7 += x39 * y7;
         a8 = A8[i+39];
         a8 += x39 * y8;
         a9 = A9[i+39];
         a9 += x39 * y9;
         a10 = A10[i+39];
         a10 += x39 * y10;
         a11 = A11[i+39];
         a11 += x39 * y11;
         a12 = A12[i+39];
         a12 += x39 * y12;
         a13 = A13[i+39];
         a13 += x39 * y13;
         a14 = A14[i+39];
         a14 += x39 * y14;
         a15 = A15[i+39];
         a15 += x39 * y15;
         a0 += w39 * z0;
         A0[i+39] = a0;
         a1 += w39 * z1;
         A1[i+39] = a1;
         a2 += w39 * z2;
         A2[i+39] = a2;
         a3 += w39 * z3;
         A3[i+39] = a3;
         a4 += w39 * z4;
         A4[i+39] = a4;
         a5 += w39 * z5;
         A5[i+39] = a5;
         a6 += w39 * z6;
         A6[i+39] = a6;
         a7 += w39 * z7;
         A7[i+39] = a7;
         a8 += w39 * z8;
         A8[i+39] = a8;
         a9 += w39 * z9;
         A9[i+39] = a9;
         a10 += w39 * z10;
         A10[i+39] = a10;
         a11 += w39 * z11;
         A11[i+39] = a11;
         a12 += w39 * z12;
         A12[i+39] = a12;
         a13 += w39 * z13;
         A13[i+39] = a13;
         a14 += w39 * z14;
         A14[i+39] = a14;
         a15 += w39 * z15;
         A15[i+39] = a15;
         a0 = A0[i+40];
         a0 += x40 * y0;
         a1 = A1[i+40];
         a1 += x40 * y1;
         a2 = A2[i+40];
         a2 += x40 * y2;
         a3 = A3[i+40];
         a3 += x40 * y3;
         a4 = A4[i+40];
         a4 += x40 * y4;
         a5 = A5[i+40];
         a5 += x40 * y5;
         a6 = A6[i+40];
         a6 += x40 * y6;
         a7 = A7[i+40];
         a7 += x40 * y7;
         a8 = A8[i+40];
         a8 += x40 * y8;
         a9 = A9[i+40];
         a9 += x40 * y9;
         a10 = A10[i+40];
         a10 += x40 * y10;
         a11 = A11[i+40];
         a11 += x40 * y11;
         a12 = A12[i+40];
         a12 += x40 * y12;
         a13 = A13[i+40];
         a13 += x40 * y13;
         a14 = A14[i+40];
         a14 += x40 * y14;
         a15 = A15[i+40];
         a15 += x40 * y15;
         a0 += w40 * z0;
         A0[i+40] = a0;
         a1 += w40 * z1;
         A1[i+40] = a1;
         a2 += w40 * z2;
         A2[i+40] = a2;
         a3 += w40 * z3;
         A3[i+40] = a3;
         a4 += w40 * z4;
         A4[i+40] = a4;
         a5 += w40 * z5;
         A5[i+40] = a5;
         a6 += w40 * z6;
         A6[i+40] = a6;
         a7 += w40 * z7;
         A7[i+40] = a7;
         a8 += w40 * z8;
         A8[i+40] = a8;
         a9 += w40 * z9;
         A9[i+40] = a9;
         a10 += w40 * z10;
         A10[i+40] = a10;
         a11 += w40 * z11;
         A11[i+40] = a11;
         a12 += w40 * z12;
         A12[i+40] = a12;
         a13 += w40 * z13;
         A13[i+40] = a13;
         a14 += w40 * z14;
         A14[i+40] = a14;
         a15 += w40 * z15;
         A15[i+40] = a15;
         a0 = A0[i+41];
         a0 += x41 * y0;
         a1 = A1[i+41];
         a1 += x41 * y1;
         a2 = A2[i+41];
         a2 += x41 * y2;
         a3 = A3[i+41];
         a3 += x41 * y3;
         a4 = A4[i+41];
         a4 += x41 * y4;
         a5 = A5[i+41];
         a5 += x41 * y5;
         a6 = A6[i+41];
         a6 += x41 * y6;
         a7 = A7[i+41];
         a7 += x41 * y7;
         a8 = A8[i+41];
         a8 += x41 * y8;
         a9 = A9[i+41];
         a9 += x41 * y9;
         a10 = A10[i+41];
         a10 += x41 * y10;
         a11 = A11[i+41];
         a11 += x41 * y11;
         a12 = A12[i+41];
         a12 += x41 * y12;
         a13 = A13[i+41];
         a13 += x41 * y13;
         a14 = A14[i+41];
         a14 += x41 * y14;
         a15 = A15[i+41];
         a15 += x41 * y15;
         a0 += w41 * z0;
         A0[i+41] = a0;
         a1 += w41 * z1;
         A1[i+41] = a1;
         a2 += w41 * z2;
         A2[i+41] = a2;
         a3 += w41 * z3;
         A3[i+41] = a3;
         a4 += w41 * z4;
         A4[i+41] = a4;
         a5 += w41 * z5;
         A5[i+41] = a5;
         a6 += w41 * z6;
         A6[i+41] = a6;
         a7 += w41 * z7;
         A7[i+41] = a7;
         a8 += w41 * z8;
         A8[i+41] = a8;
         a9 += w41 * z9;
         A9[i+41] = a9;
         a10 += w41 * z10;
         A10[i+41] = a10;
         a11 += w41 * z11;
         A11[i+41] = a11;
         a12 += w41 * z12;
         A12[i+41] = a12;
         a13 += w41 * z13;
         A13[i+41] = a13;
         a14 += w41 * z14;
         A14[i+41] = a14;
         a15 += w41 * z15;
         A15[i+41] = a15;
         a0 = A0[i+42];
         a0 += x42 * y0;
         a1 = A1[i+42];
         a1 += x42 * y1;
         a2 = A2[i+42];
         a2 += x42 * y2;
         a3 = A3[i+42];
         a3 += x42 * y3;
         a4 = A4[i+42];
         a4 += x42 * y4;
         a5 = A5[i+42];
         a5 += x42 * y5;
         a6 = A6[i+42];
         a6 += x42 * y6;
         a7 = A7[i+42];
         a7 += x42 * y7;
         a8 = A8[i+42];
         a8 += x42 * y8;
         a9 = A9[i+42];
         a9 += x42 * y9;
         a10 = A10[i+42];
         a10 += x42 * y10;
         a11 = A11[i+42];
         a11 += x42 * y11;
         a12 = A12[i+42];
         a12 += x42 * y12;
         a13 = A13[i+42];
         a13 += x42 * y13;
         a14 = A14[i+42];
         a14 += x42 * y14;
         a15 = A15[i+42];
         a15 += x42 * y15;
         a0 += w42 * z0;
         A0[i+42] = a0;
         a1 += w42 * z1;
         A1[i+42] = a1;
         a2 += w42 * z2;
         A2[i+42] = a2;
         a3 += w42 * z3;
         A3[i+42] = a3;
         a4 += w42 * z4;
         A4[i+42] = a4;
         a5 += w42 * z5;
         A5[i+42] = a5;
         a6 += w42 * z6;
         A6[i+42] = a6;
         a7 += w42 * z7;
         A7[i+42] = a7;
         a8 += w42 * z8;
         A8[i+42] = a8;
         a9 += w42 * z9;
         A9[i+42] = a9;
         a10 += w42 * z10;
         A10[i+42] = a10;
         a11 += w42 * z11;
         A11[i+42] = a11;
         a12 += w42 * z12;
         A12[i+42] = a12;
         a13 += w42 * z13;
         A13[i+42] = a13;
         a14 += w42 * z14;
         A14[i+42] = a14;
         a15 += w42 * z15;
         A15[i+42] = a15;
         a0 = A0[i+43];
         a0 += x43 * y0;
         a1 = A1[i+43];
         a1 += x43 * y1;
         a2 = A2[i+43];
         a2 += x43 * y2;
         a3 = A3[i+43];
         a3 += x43 * y3;
         a4 = A4[i+43];
         a4 += x43 * y4;
         a5 = A5[i+43];
         a5 += x43 * y5;
         a6 = A6[i+43];
         a6 += x43 * y6;
         a7 = A7[i+43];
         a7 += x43 * y7;
         a8 = A8[i+43];
         a8 += x43 * y8;
         a9 = A9[i+43];
         a9 += x43 * y9;
         a10 = A10[i+43];
         a10 += x43 * y10;
         a11 = A11[i+43];
         a11 += x43 * y11;
         a12 = A12[i+43];
         a12 += x43 * y12;
         a13 = A13[i+43];
         a13 += x43 * y13;
         a14 = A14[i+43];
         a14 += x43 * y14;
         a15 = A15[i+43];
         a15 += x43 * y15;
         a0 += w43 * z0;
         A0[i+43] = a0;
         a1 += w43 * z1;
         A1[i+43] = a1;
         a2 += w43 * z2;
         A2[i+43] = a2;
         a3 += w43 * z3;
         A3[i+43] = a3;
         a4 += w43 * z4;
         A4[i+43] = a4;
         a5 += w43 * z5;
         A5[i+43] = a5;
         a6 += w43 * z6;
         A6[i+43] = a6;
         a7 += w43 * z7;
         A7[i+43] = a7;
         a8 += w43 * z8;
         A8[i+43] = a8;
         a9 += w43 * z9;
         A9[i+43] = a9;
         a10 += w43 * z10;
         A10[i+43] = a10;
         a11 += w43 * z11;
         A11[i+43] = a11;
         a12 += w43 * z12;
         A12[i+43] = a12;
         a13 += w43 * z13;
         A13[i+43] = a13;
         a14 += w43 * z14;
         A14[i+43] = a14;
         a15 += w43 * z15;
         A15[i+43] = a15;
         a0 = A0[i+44];
         a0 += x44 * y0;
         a1 = A1[i+44];
         a1 += x44 * y1;
         a2 = A2[i+44];
         a2 += x44 * y2;
         a3 = A3[i+44];
         a3 += x44 * y3;
         a4 = A4[i+44];
         a4 += x44 * y4;
         a5 = A5[i+44];
         a5 += x44 * y5;
         a6 = A6[i+44];
         a6 += x44 * y6;
         a7 = A7[i+44];
         a7 += x44 * y7;
         a8 = A8[i+44];
         a8 += x44 * y8;
         a9 = A9[i+44];
         a9 += x44 * y9;
         a10 = A10[i+44];
         a10 += x44 * y10;
         a11 = A11[i+44];
         a11 += x44 * y11;
         a12 = A12[i+44];
         a12 += x44 * y12;
         a13 = A13[i+44];
         a13 += x44 * y13;
         a14 = A14[i+44];
         a14 += x44 * y14;
         a15 = A15[i+44];
         a15 += x44 * y15;
         a0 += w44 * z0;
         A0[i+44] = a0;
         a1 += w44 * z1;
         A1[i+44] = a1;
         a2 += w44 * z2;
         A2[i+44] = a2;
         a3 += w44 * z3;
         A3[i+44] = a3;
         a4 += w44 * z4;
         A4[i+44] = a4;
         a5 += w44 * z5;
         A5[i+44] = a5;
         a6 += w44 * z6;
         A6[i+44] = a6;
         a7 += w44 * z7;
         A7[i+44] = a7;
         a8 += w44 * z8;
         A8[i+44] = a8;
         a9 += w44 * z9;
         A9[i+44] = a9;
         a10 += w44 * z10;
         A10[i+44] = a10;
         a11 += w44 * z11;
         A11[i+44] = a11;
         a12 += w44 * z12;
         A12[i+44] = a12;
         a13 += w44 * z13;
         A13[i+44] = a13;
         a14 += w44 * z14;
         A14[i+44] = a14;
         a15 += w44 * z15;
         A15[i+44] = a15;
         a0 = A0[i+45];
         a0 += x45 * y0;
         a1 = A1[i+45];
         a1 += x45 * y1;
         a2 = A2[i+45];
         a2 += x45 * y2;
         a3 = A3[i+45];
         a3 += x45 * y3;
         a4 = A4[i+45];
         a4 += x45 * y4;
         a5 = A5[i+45];
         a5 += x45 * y5;
         a6 = A6[i+45];
         a6 += x45 * y6;
         a7 = A7[i+45];
         a7 += x45 * y7;
         a8 = A8[i+45];
         a8 += x45 * y8;
         a9 = A9[i+45];
         a9 += x45 * y9;
         a10 = A10[i+45];
         a10 += x45 * y10;
         a11 = A11[i+45];
         a11 += x45 * y11;
         a12 = A12[i+45];
         a12 += x45 * y12;
         a13 = A13[i+45];
         a13 += x45 * y13;
         a14 = A14[i+45];
         a14 += x45 * y14;
         a15 = A15[i+45];
         a15 += x45 * y15;
         a0 += w45 * z0;
         A0[i+45] = a0;
         a1 += w45 * z1;
         A1[i+45] = a1;
         a2 += w45 * z2;
         A2[i+45] = a2;
         a3 += w45 * z3;
         A3[i+45] = a3;
         a4 += w45 * z4;
         A4[i+45] = a4;
         a5 += w45 * z5;
         A5[i+45] = a5;
         a6 += w45 * z6;
         A6[i+45] = a6;
         a7 += w45 * z7;
         A7[i+45] = a7;
         a8 += w45 * z8;
         A8[i+45] = a8;
         a9 += w45 * z9;
         A9[i+45] = a9;
         a10 += w45 * z10;
         A10[i+45] = a10;
         a11 += w45 * z11;
         A11[i+45] = a11;
         a12 += w45 * z12;
         A12[i+45] = a12;
         a13 += w45 * z13;
         A13[i+45] = a13;
         a14 += w45 * z14;
         A14[i+45] = a14;
         a15 += w45 * z15;
         A15[i+45] = a15;
         a0 = A0[i+46];
         a0 += x46 * y0;
         a1 = A1[i+46];
         a1 += x46 * y1;
         a2 = A2[i+46];
         a2 += x46 * y2;
         a3 = A3[i+46];
         a3 += x46 * y3;
         a4 = A4[i+46];
         a4 += x46 * y4;
         a5 = A5[i+46];
         a5 += x46 * y5;
         a6 = A6[i+46];
         a6 += x46 * y6;
         a7 = A7[i+46];
         a7 += x46 * y7;
         a8 = A8[i+46];
         a8 += x46 * y8;
         a9 = A9[i+46];
         a9 += x46 * y9;
         a10 = A10[i+46];
         a10 += x46 * y10;
         a11 = A11[i+46];
         a11 += x46 * y11;
         a12 = A12[i+46];
         a12 += x46 * y12;
         a13 = A13[i+46];
         a13 += x46 * y13;
         a14 = A14[i+46];
         a14 += x46 * y14;
         a15 = A15[i+46];
         a15 += x46 * y15;
         a0 += w46 * z0;
         A0[i+46] = a0;
         a1 += w46 * z1;
         A1[i+46] = a1;
         a2 += w46 * z2;
         A2[i+46] = a2;
         a3 += w46 * z3;
         A3[i+46] = a3;
         a4 += w46 * z4;
         A4[i+46] = a4;
         a5 += w46 * z5;
         A5[i+46] = a5;
         a6 += w46 * z6;
         A6[i+46] = a6;
         a7 += w46 * z7;
         A7[i+46] = a7;
         a8 += w46 * z8;
         A8[i+46] = a8;
         a9 += w46 * z9;
         A9[i+46] = a9;
         a10 += w46 * z10;
         A10[i+46] = a10;
         a11 += w46 * z11;
         A11[i+46] = a11;
         a12 += w46 * z12;
         A12[i+46] = a12;
         a13 += w46 * z13;
         A13[i+46] = a13;
         a14 += w46 * z14;
         A14[i+46] = a14;
         a15 += w46 * z15;
         A15[i+46] = a15;
         a0 = A0[i+47];
         a0 += x47 * y0;
         a1 = A1[i+47];
         a1 += x47 * y1;
         a2 = A2[i+47];
         a2 += x47 * y2;
         a3 = A3[i+47];
         a3 += x47 * y3;
         a4 = A4[i+47];
         a4 += x47 * y4;
         a5 = A5[i+47];
         a5 += x47 * y5;
         a6 = A6[i+47];
         a6 += x47 * y6;
         a7 = A7[i+47];
         a7 += x47 * y7;
         a8 = A8[i+47];
         a8 += x47 * y8;
         a9 = A9[i+47];
         a9 += x47 * y9;
         a10 = A10[i+47];
         a10 += x47 * y10;
         a11 = A11[i+47];
         a11 += x47 * y11;
         a12 = A12[i+47];
         a12 += x47 * y12;
         a13 = A13[i+47];
         a13 += x47 * y13;
         a14 = A14[i+47];
         a14 += x47 * y14;
         a15 = A15[i+47];
         a15 += x47 * y15;
         a0 += w47 * z0;
         A0[i+47] = a0;
         a1 += w47 * z1;
         A1[i+47] = a1;
         a2 += w47 * z2;
         A2[i+47] = a2;
         a3 += w47 * z3;
         A3[i+47] = a3;
         a4 += w47 * z4;
         A4[i+47] = a4;
         a5 += w47 * z5;
         A5[i+47] = a5;
         a6 += w47 * z6;
         A6[i+47] = a6;
         a7 += w47 * z7;
         A7[i+47] = a7;
         a8 += w47 * z8;
         A8[i+47] = a8;
         a9 += w47 * z9;
         A9[i+47] = a9;
         a10 += w47 * z10;
         A10[i+47] = a10;
         a11 += w47 * z11;
         A11[i+47] = a11;
         a12 += w47 * z12;
         A12[i+47] = a12;
         a13 += w47 * z13;
         A13[i+47] = a13;
         a14 += w47 * z14;
         A14[i+47] = a14;
         a15 += w47 * z15;
         A15[i+47] = a15;

      }
      for (i=M48; i < M; i++)
      {
         const register float x0 = X[i], w0 = W[i];
         register float a0=A0[i], a1=A1[i], a2=A2[i], a3=A3[i], a4=A4[i],
                        a5=A5[i], a6=A6[i], a7=A7[i], a8=A8[i], a9=A9[i],
                        a10=A10[i], a11=A11[i], a12=A12[i], a13=A13[i],
                        a14=A14[i], a15=A15[i];

         a0 += x0 * y0;
         a1 += x0 * y1;
         a2 += x0 * y2;
         a3 += x0 * y3;
         a4 += x0 * y4;
         a5 += x0 * y5;
         a6 += x0 * y6;
         a7 += x0 * y7;
         a8 += x0 * y8;
         a9 += x0 * y9;
         a10 += x0 * y10;
         a11 += x0 * y11;
         a12 += x0 * y12;
         a13 += x0 * y13;
         a14 += x0 * y14;
         a15 += x0 * y15;
         a0 += w0 * z0;
         A0[i] = a0;
         a1 += w0 * z1;
         A1[i] = a1;
         a2 += w0 * z2;
         A2[i] = a2;
         a3 += w0 * z3;
         A3[i] = a3;
         a4 += w0 * z4;
         A4[i] = a4;
         a5 += w0 * z5;
         A5[i] = a5;
         a6 += w0 * z6;
         A6[i] = a6;
         a7 += w0 * z7;
         A7[i] = a7;
         a8 += w0 * z8;
         A8[i] = a8;
         a9 += w0 * z9;
         A9[i] = a9;
         a10 += w0 * z10;
         A10[i] = a10;
         a11 += w0 * z11;
         A11[i] = a11;
         a12 += w0 * z12;
         A12[i] = a12;
         a13 += w0 * z13;
         A13[i] = a13;
         a14 += w0 * z14;
         A14[i] = a14;
         a15 += w0 * z15;
         A15[i] = a15;
      }
      Z += 16;
   }
/*
 * Do remaining columns with NU=1 cleanup
 */
   for (j=N-N16; j; j--, A += lda, Y++)
   {
      float *A0=A;
      const register float y0=Y[0], z0=Z[0];
      ATL_INT i;
      prefY(Y+1+1-1);
      prefY(Z+1+1-1);
      for (i=0; i < M48; i += 48)
      {
         register float a0;
         const float x0=X[i+0], w0=W[i+0], x1=X[i+1], w1=W[i+1], x2=X[i+2],
                     w2=W[i+2], x3=X[i+3], w3=W[i+3], x4=X[i+4], w4=W[i+4],
                     x5=X[i+5], w5=W[i+5], x6=X[i+6], w6=W[i+6], x7=X[i+7],
                     w7=W[i+7], x8=X[i+8], w8=W[i+8], x9=X[i+9], w9=W[i+9],
                     x10=X[i+10], w10=W[i+10], x11=X[i+11], w11=W[i+11],
                     x12=X[i+12], w12=W[i+12], x13=X[i+13], w13=W[i+13],
                     x14=X[i+14], w14=W[i+14], x15=X[i+15], w15=W[i+15],
                     x16=X[i+16], w16=W[i+16], x17=X[i+17], w17=W[i+17],
                     x18=X[i+18], w18=W[i+18], x19=X[i+19], w19=W[i+19],
                     x20=X[i+20], w20=W[i+20], x21=X[i+21], w21=W[i+21],
                     x22=X[i+22], w22=W[i+22], x23=X[i+23], w23=W[i+23],
                     x24=X[i+24], w24=W[i+24], x25=X[i+25], w25=W[i+25],
                     x26=X[i+26], w26=W[i+26], x27=X[i+27], w27=W[i+27],
                     x28=X[i+28], w28=W[i+28], x29=X[i+29], w29=W[i+29],
                     x30=X[i+30], w30=W[i+30], x31=X[i+31], w31=W[i+31],
                     x32=X[i+32], w32=W[i+32], x33=X[i+33], w33=W[i+33],
                     x34=X[i+34], w34=W[i+34], x35=X[i+35], w35=W[i+35],
                     x36=X[i+36], w36=W[i+36], x37=X[i+37], w37=W[i+37],
                     x38=X[i+38], w38=W[i+38], x39=X[i+39], w39=W[i+39],
                     x40=X[i+40], w40=W[i+40], x41=X[i+41], w41=W[i+41],
                     x42=X[i+42], w42=W[i+42], x43=X[i+43], w43=W[i+43],
                     x44=X[i+44], w44=W[i+44], x45=X[i+45], w45=W[i+45],
                     x46=X[i+46], w46=W[i+46], x47=X[i+47], w47=W[i+47];

         prefX(X);
         a0 = A0[i+0];
         a0 += x0 * y0;
         prefX(W);
         a0 += w0 * z0;
         prefA(A0);
         A0[i+0] = a0;
         a0 = A0[i+1];
         a0 += x1 * y0;
         a0 += w1 * z0;
         A0[i+1] = a0;
         a0 = A0[i+2];
         a0 += x2 * y0;
         a0 += w2 * z0;
         A0[i+2] = a0;
         a0 = A0[i+3];
         a0 += x3 * y0;
         a0 += w3 * z0;
         A0[i+3] = a0;
         a0 = A0[i+4];
         a0 += x4 * y0;
         a0 += w4 * z0;
         A0[i+4] = a0;
         a0 = A0[i+5];
         a0 += x5 * y0;
         a0 += w5 * z0;
         A0[i+5] = a0;
         a0 = A0[i+6];
         a0 += x6 * y0;
         a0 += w6 * z0;
         A0[i+6] = a0;
         a0 = A0[i+7];
         a0 += x7 * y0;
         a0 += w7 * z0;
         A0[i+7] = a0;
         a0 = A0[i+8];
         a0 += x8 * y0;
         a0 += w8 * z0;
         A0[i+8] = a0;
         a0 = A0[i+9];
         a0 += x9 * y0;
         a0 += w9 * z0;
         A0[i+9] = a0;
         a0 = A0[i+10];
         a0 += x10 * y0;
         a0 += w10 * z0;
         A0[i+10] = a0;
         a0 = A0[i+11];
         a0 += x11 * y0;
         a0 += w11 * z0;
         A0[i+11] = a0;
         a0 = A0[i+12];
         a0 += x12 * y0;
         a0 += w12 * z0;
         A0[i+12] = a0;
         a0 = A0[i+13];
         a0 += x13 * y0;
         a0 += w13 * z0;
         A0[i+13] = a0;
         a0 = A0[i+14];
         a0 += x14 * y0;
         a0 += w14 * z0;
         A0[i+14] = a0;
         a0 = A0[i+15];
         a0 += x15 * y0;
         a0 += w15 * z0;
         A0[i+15] = a0;
         a0 = A0[i+16];
         a0 += x16 * y0;
         a0 += w16 * z0;
         prefA(A0);
         A0[i+16] = a0;
         a0 = A0[i+17];
         a0 += x17 * y0;
         a0 += w17 * z0;
         A0[i+17] = a0;
         a0 = A0[i+18];
         a0 += x18 * y0;
         a0 += w18 * z0;
         A0[i+18] = a0;
         a0 = A0[i+19];
         a0 += x19 * y0;
         a0 += w19 * z0;
         A0[i+19] = a0;
         a0 = A0[i+20];
         a0 += x20 * y0;
         a0 += w20 * z0;
         A0[i+20] = a0;
         a0 = A0[i+21];
         a0 += x21 * y0;
         a0 += w21 * z0;
         A0[i+21] = a0;
         a0 = A0[i+22];
         a0 += x22 * y0;
         a0 += w22 * z0;
         A0[i+22] = a0;
         a0 = A0[i+23];
         a0 += x23 * y0;
         a0 += w23 * z0;
         A0[i+23] = a0;
         a0 = A0[i+24];
         a0 += x24 * y0;
         a0 += w24 * z0;
         A0[i+24] = a0;
         a0 = A0[i+25];
         a0 += x25 * y0;
         a0 += w25 * z0;
         A0[i+25] = a0;
         a0 = A0[i+26];
         a0 += x26 * y0;
         a0 += w26 * z0;
         A0[i+26] = a0;
         a0 = A0[i+27];
         a0 += x27 * y0;
         a0 += w27 * z0;
         A0[i+27] = a0;
         a0 = A0[i+28];
         a0 += x28 * y0;
         a0 += w28 * z0;
         A0[i+28] = a0;
         a0 = A0[i+29];
         a0 += x29 * y0;
         a0 += w29 * z0;
         A0[i+29] = a0;
         a0 = A0[i+30];
         a0 += x30 * y0;
         a0 += w30 * z0;
         A0[i+30] = a0;
         a0 = A0[i+31];
         a0 += x31 * y0;
         a0 += w31 * z0;
         A0[i+31] = a0;
         a0 = A0[i+32];
         a0 += x32 * y0;
         a0 += w32 * z0;
         prefA(A0);
         A0[i+32] = a0;
         a0 = A0[i+33];
         a0 += x33 * y0;
         a0 += w33 * z0;
         A0[i+33] = a0;
         a0 = A0[i+34];
         a0 += x34 * y0;
         a0 += w34 * z0;
         A0[i+34] = a0;
         a0 = A0[i+35];
         a0 += x35 * y0;
         a0 += w35 * z0;
         A0[i+35] = a0;
         a0 = A0[i+36];
         a0 += x36 * y0;
         a0 += w36 * z0;
         A0[i+36] = a0;
         a0 = A0[i+37];
         a0 += x37 * y0;
         a0 += w37 * z0;
         A0[i+37] = a0;
         a0 = A0[i+38];
         a0 += x38 * y0;
         a0 += w38 * z0;
         A0[i+38] = a0;
         a0 = A0[i+39];
         a0 += x39 * y0;
         a0 += w39 * z0;
         A0[i+39] = a0;
         a0 = A0[i+40];
         a0 += x40 * y0;
         a0 += w40 * z0;
         A0[i+40] = a0;
         a0 = A0[i+41];
         a0 += x41 * y0;
         a0 += w41 * z0;
         A0[i+41] = a0;
         a0 = A0[i+42];
         a0 += x42 * y0;
         a0 += w42 * z0;
         A0[i+42] = a0;
         a0 = A0[i+43];
         a0 += x43 * y0;
         a0 += w43 * z0;
         A0[i+43] = a0;
         a0 = A0[i+44];
         a0 += x44 * y0;
         a0 += w44 * z0;
         A0[i+44] = a0;
         a0 = A0[i+45];
         a0 += x45 * y0;
         a0 += w45 * z0;
         A0[i+45] = a0;
         a0 = A0[i+46];
         a0 += x46 * y0;
         a0 += w46 * z0;
         A0[i+46] = a0;
         a0 = A0[i+47];
         a0 += x47 * y0;
         a0 += w47 * z0;
         A0[i+47] = a0;

      }
      for (i=M48; i < M; i++)
      {
         const register float x0 = X[i], w0 = W[i];
         register float a0=A0[i];

         a0 += x0 * y0;
         a0 += w0 * z0;
         A0[i] = a0;
      }
      Z++;
   }
}
